@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2018 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import os.path
@@ -44,8 +44,16 @@ class IAdminPanelProvider(Interface):
         """Process a request for an admin panel.
 
         This function should return a tuple of the form `(template, data)`,
-        where `template` is the name of the template to use and `data` is the
-        data to be passed to the template.
+        where `template` is the name of the template to use and `data` is
+        the data to use when rendering the template.
+
+        .. note::
+
+           When a plugin wants to use a legacy Genshi template instead
+           of a Jinja2 template, it needs to return instead a *triple*
+           of the form ``(template, data, None)``, similar to what
+           `.IRequestHandler.process_request` does.
+
         """
 
 
@@ -66,18 +74,18 @@ class IAdminCommandProvider(Interface):
         """Return a list of available admin commands.
 
         The items returned by this function must be tuples of the form
-        `(command, args, help, complete, execute)`, where `command` contains
-        the space-separated command and sub-command names, `args` is a string
-        describing the command arguments and `help` is the help text. The
-        first paragraph of the help text is taken as a short help, shown in the
-        list of commands.
+        `(command, args, help, complete, execute)`, where `command`
+        contains the space-separated command and sub-command names, `args`
+        is a string describing the command arguments and `help` is the
+        help text. The first paragraph of the help text is taken as a
+        short help, shown in the list of commands.
 
-        `complete` is called to auto-complete the command arguments, with the
-        current list of arguments as its only argument. It should return a list
-        of relevant values for the last argument in the list.
+        `complete` is called to auto-complete the command arguments, with
+        the current list of arguments as its only argument. It should
+        return a list of relevant values for the last argument in the list.
 
-        `execute` is called to execute the command, with the command arguments
-        passed as positional arguments.
+        `execute` is called to execute the command, with the command
+        arguments passed as positional arguments.
         """
 
 
@@ -104,11 +112,11 @@ class AdminCommandManager(Component):
             for cmd in provider.get_admin_commands() or []:
                 parts = cmd[0].split()
                 plen = min(len(parts), len(args) - 1)
-                if args[:plen] != parts[:plen]:         # Prefix doesn't match
+                if args[:plen] != parts[:plen]:        # Prefix doesn't match
                     continue
-                elif len(args) <= len(parts):           # Command name
+                elif len(args) <= len(parts):          # Command name
                     comp.append(parts[len(args) - 1])
-                elif not cmd_only:                      # Arguments
+                elif not cmd_only:                     # Arguments
                     if cmd[3] is None:
                         return []
                     return cmd[3](args[len(parts):]) or []
@@ -128,7 +136,7 @@ class AdminCommandManager(Component):
                     except AdminCommandError as e:
                         e.cmd = ' '.join(parts)
                         raise
-                    except TypeError as e:
+                    except TypeError:
                         tb = traceback.extract_tb(sys.exc_info()[2])
                         if len(tb) == 1:
                             raise AdminCommandError(_("Invalid arguments"),
@@ -144,7 +152,7 @@ class AdminCommandManager(Component):
         cmds = set()
         for provider in self.providers:
             for cmd in provider.get_admin_commands() or []:
-                cmds.add(cmd[0].split()[0]) # use only first token
+                cmds.add(cmd[0].split()[0])  # use only first token
 
         def score(cmd, arg):
             if cmd.startswith(arg):
@@ -158,7 +166,7 @@ class AdminCommandManager(Component):
 class PrefixList(list):
     """A list of prefixes for command argument auto-completion."""
     def complete(self, text):
-        return list(set(a for a in self if a.startswith(text)))
+        return list({a for a in self if a.startswith(text)})
 
 
 def path_startswith(path, prefix):
@@ -169,7 +177,7 @@ class PathList(list):
     """A list of paths for command argument auto-completion."""
     def complete(self, text):
         """Return the items in the list matching text."""
-        matches = list(set(a for a in self if path_startswith(a, text)))
+        matches = list({a for a in self if path_startswith(a, text)})
         if len(matches) == 1 and not os.path.isdir(matches[0]):
             matches[0] += ' '
         return matches
