@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -18,11 +18,13 @@ from glob import glob
 import imp
 import os.path
 import pkg_resources
-from pkg_resources import working_set, DistributionNotFound, VersionConflict, \
-                          UnknownExtra
+from pkg_resources import working_set, DistributionNotFound, \
+                          VersionConflict, UnknownExtra
 import sys
 
-from trac.util import get_doc, get_module_path, get_sources, get_pkginfo
+from trac.core import ComponentMeta
+from trac.util import get_doc, get_module_metadata, get_module_path, \
+                      get_pkginfo, get_sources
 from trac.util.text import exception_to_unicode, to_unicode
 
 __all__ = ['load_components']
@@ -33,6 +35,7 @@ def _enable_plugin(env, module):
     if env.is_component_enabled(module) is None:
         env.enable_component(module)
 
+
 def load_eggs(entry_point_name):
     """Loader that loads any eggs on the search path and `sys.path`."""
     def _load_eggs(env, search_path, auto_enable=None):
@@ -42,18 +45,16 @@ def load_eggs(entry_point_name):
         )
         for dist in distributions:
             if dist not in working_set:
-                env.log.debug('Adding plugin %s from %s', dist, dist.location)
+                env.log.debug('Adding plugin "%s" from "%s"',
+                              dist, dist.location)
                 working_set.add(dist)
 
         def _log_error(item, e):
             ue = exception_to_unicode(e)
             if isinstance(e, DistributionNotFound):
-                env.log.debug('Skipping "%s": ("%s" not found)', item, ue)
-            elif isinstance(e, VersionConflict):
-                env.log.error('Skipping "%s": (version conflict "%s")',
-                              item, ue)
-            elif isinstance(e, UnknownExtra):
-                env.log.error('Skipping "%s": (unknown extra "%s")', item, ue)
+                env.log.debug('Skipping "%s": %s', item, ue)
+            elif isinstance(e, (ImportError, UnknownExtra, VersionConflict)):
+                env.log.error('Skipping "%s": %s', item, ue)
             else:
                 env.log.error('Skipping "%s": %s', item,
                               exception_to_unicode(e, traceback=True))
@@ -61,18 +62,29 @@ def load_eggs(entry_point_name):
         for dist, e in errors.iteritems():
             _log_error(dist, e)
 
+        def deregister_components(entry_point):
+            """Remove components for `entry_point` from the registry."""
+            for name in entry_point.attrs:
+                for c in ComponentMeta._components:
+                    if c.__module__ == entry_point.module_name and \
+                            c.__name__ == name:
+                        ComponentMeta.deregister(c)
+
         for entry in sorted(working_set.iter_entry_points(entry_point_name),
                             key=lambda entry: entry.name):
-            env.log.debug('Loading %s from %s', entry.name, entry.dist.location)
+            env.log.debug('Loading plugin "%s" from "%s"',
+                          entry.name, entry.dist.location)
             try:
                 entry.load(require=True)
-            except Exception, e:
+            except Exception as e:
                 _log_error(entry, e)
+                deregister_components(entry)
             else:
                 if os.path.dirname(entry.dist.location) == auto_enable:
                     _enable_plugin(env, entry.module_name)
     return _load_eggs
 
+
 def load_py_files():
     """Loader that look for Python source files in the plugins directories,
     which simply get imported, thereby registering them with the component
@@ -82,30 +94,26 @@ def load_py_files():
         for path in search_path:
             plugin_files = glob(os.path.join(path, '*.py'))
             for plugin_file in plugin_files:
+                plugin_name = os.path.basename(plugin_file[:-3])
+                env.log.debug("Loading file plugin %s from %s",
+                              plugin_name, plugin_file)
                 try:
-                    plugin_name = os.path.basename(plugin_file[:-3])
-                    env.log.debug('Loading file plugin %s from %s',
-                                  plugin_name, plugin_file)
                     if plugin_name not in sys.modules:
-                        module = imp.load_source(plugin_name, plugin_file)
+                        imp.load_source(plugin_name, plugin_file)
+                except (ImportError, VersionConflict) as e:
+                    env.log.error('Skipping "%s": %s', plugin_name,
+                                  exception_to_unicode(e))
+                except (Exception, SystemExit) as e:
+                    env.log.error(
+                        "Failed to load plugin from %s: %s", plugin_file,
+                        exception_to_unicode(e, traceback=True))
+                else:
                     if path == auto_enable:
                         _enable_plugin(env, plugin_name)
-                except Exception, e:
-                    env.log.error('Failed to load plugin from %s: %s',
-                                  plugin_file,
-                                  exception_to_unicode(e, traceback=True))
 
     return _load_py_files
 
 
-def get_plugins_dir(env):
-    """Return the path to the `plugins` directory of the environment.
-
-    :since 1.0.11: Deprecated and will be removed in 1.3.1. Use the
-                   Environment.plugins_dir property instead."""
-    return env.plugins_dir
-
-
 def load_components(env, extra_path=None, loaders=(load_eggs('trac.plugins'),
                                                    load_py_files())):
     """Load all plugin components found on the given search path."""
@@ -140,7 +148,6 @@ def get_plugin_info(env, include_core=False):
 
     plugins_dir = env.plugins_dir
     plugins = {}
-    from trac.core import ComponentMeta
     for component in ComponentMeta._components:
         module = sys.modules[component.__module__]
 
@@ -156,19 +163,8 @@ def get_plugin_info(env, include_core=False):
                 readonly = False
             # retrieve plugin metadata
             info = get_pkginfo(dist)
-            if not info:
-                info = {}
-                for k in ('author', 'author_email', 'home_page', 'url',
-                          'license', 'trac'):
-                    v = getattr(module, k, '')
-                    if v and isinstance(v, basestring):
-                        if k == 'home_page' or k == 'url':
-                            k = 'home_page'
-                            v = v.replace('$', '').replace('URL: ', '')
-                        else:
-                            v = to_unicode(v)
-                        info[k] = v
-            else:
+            version = dist.version
+            if info:
                 # Info found; set all those fields to "None" that have the
                 # value "UNKNOWN" as this is the value for fields that
                 # aren't specified in "setup.py"
@@ -179,17 +175,10 @@ def get_plugin_info(env, include_core=False):
                         # Must be encoded as unicode as otherwise Genshi
                         # may raise a "UnicodeDecodeError".
                         info[k] = to_unicode(info[k])
+            else:
+                info = get_module_metadata(module)
+                version = info['version']
 
-            # retrieve plugin version info
-            version = dist.version
-            if not version:
-                version = (getattr(module, 'version', '') or
-                           getattr(module, 'revision', ''))
-                # special handling for "$Rev$" strings
-                if version != '$Rev$':
-                    version = version.replace('$', '').replace('Rev: ', 'r')
-                else:  # keyword hasn't been expanded
-                    version = ''
             plugins[dist.project_name] = {
                 'name': dist.project_name, 'version': version,
                 'path': dist.location, 'plugin_filename': plugin_filename,
@@ -214,7 +203,7 @@ def get_plugin_info(env, include_core=False):
             'required': getattr(c, 'required', False),
         }
     if not include_core:
-        for name in plugins.keys():
+        for name in list(plugins):
             if name.lower() == 'trac':
                 plugins.pop(name)
     return sorted(plugins.itervalues(),
