@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Ludvig Strigeus
@@ -22,7 +22,7 @@ import re
 from trac.core import *
 from trac.mimeview.api import content_to_unicode, IHTMLPreviewRenderer, \
                               Mimeview
-from trac.util.html import escape, Markup
+from trac.util.html import Markup, escape
 from trac.util.text import expandtabs
 from trac.util.translation import _
 from trac.web.chrome import Chrome, add_script, add_stylesheet
@@ -59,8 +59,7 @@ class PatchRenderer(Component):
 
         add_script(req, 'common/js/diff.js')
         add_stylesheet(req, 'common/css/diff.css')
-        return Chrome(self.env).render_template(req, 'diff_div.html',
-                                                data, fragment=True)
+        return Chrome(self.env).render_fragment(req, 'diff_div.html', data)
 
     # Internal methods
 
@@ -91,19 +90,19 @@ class PatchRenderer(Component):
         space_re = re.compile(' ( +)|^ ')
         def htmlify(match):
             div, mod = divmod(len(match.group(0)), 2)
-            return div * '&nbsp; ' + mod * '&nbsp;'
+            return Markup(div * '&nbsp; ' + mod * '&nbsp;')
 
         comments = []
         changes = []
         lines = iter(difflines)
         try:
-            line = lines.next()
+            line = next(lines)
             while True:
                 oldpath = oldrev = newpath = newrev = ''
                 oldinfo = newinfo = []
                 binary = False
 
-                # consume preample, storing free lines in comments
+                # consume preamble, storing free lines in comments
                 # (also detect the special case of git binary patches)
                 if not line.startswith('--- '):
                     if not line.startswith('Index: ') and line != '=' * 67:
@@ -123,12 +122,12 @@ class PatchRenderer(Component):
                             oldrev, newrev = index[0].split()[-1].split('..')
                             oldinfo.append(oldrev)
                             newinfo.append(newrev)
-                        line = lines.next()
+                        line = next(lines)
                         while line:
                             comments.append(line)
-                            line = lines.next()
+                            line = next(lines)
                     else:
-                        line = lines.next()
+                        line = next(lines)
                         continue
 
                 if not oldinfo and not newinfo:
@@ -140,7 +139,7 @@ class PatchRenderer(Component):
                             oldrev = oldinfo[2]
 
                     # Changed filename/version from '+++ <file> [rev]'
-                    line = lines.next()
+                    line = next(lines)
                     if not line.startswith('+++ '):
                         self.log.debug('expected +++ after ---, got %s', line)
                         return None
@@ -195,7 +194,7 @@ class PatchRenderer(Component):
                                         'rev': ' '.join(newinfo[1:]),
                                         'shortrev': shortrev[1]}})
                 comments = []
-                line = lines.next()
+                line = next(lines)
                 while line:
                     # "@@ -333,10 +329,8 @@" or "@@ -1 +1 @@ [... title ...]"
                     r = re.match(r'@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@'
@@ -211,7 +210,7 @@ class PatchRenderer(Component):
 
                     fromend += fromline
                     toend += toline
-                    line = lines.next()
+                    line = next(lines)
                     while fromline < fromend or toline < toend or extra:
 
                         # First character is the command
@@ -258,7 +257,7 @@ class PatchRenderer(Component):
                             else:
                                 toline += 1
                             block[side]['lines'].append(line)
-                        line = lines.next()
+                        line = next(lines)
                         extra = line and line[0] == '\\'
         except StopIteration:
             pass
@@ -280,8 +279,8 @@ class PatchRenderer(Component):
                     for i in xrange(len(f)):
                         line = expandtabs(f[i], tabwidth, '\0\1')
                         line = escape(line, quotes=False)
-                        line = '<del>'.join([space_re.sub(htmlify, seg)
-                                             for seg in line.split('\0')])
+                        line = '<del>'.join(space_re.sub(htmlify, seg)
+                                            for seg in line.split('\0'))
                         line = line.replace('\1', '</del>')
                         f[i] = Markup(line)
                         if 'meta' in base and i in base['meta']:
@@ -289,8 +288,8 @@ class PatchRenderer(Component):
                     for i in xrange(len(t)):
                         line = expandtabs(t[i], tabwidth, '\0\1')
                         line = escape(line, quotes=False)
-                        line = '<ins>'.join([space_re.sub(htmlify, seg)
-                                             for seg in line.split('\0')])
+                        line = '<ins>'.join(space_re.sub(htmlify, seg)
+                                            for seg in line.split('\0'))
                         line = line.replace('\1', '</ins>')
                         t[i] = Markup(line)
                         if 'meta' in changed and i in changed['meta']:
