@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -37,12 +37,8 @@ __all__ = ['IPermissionRequestor', 'IPermissionStore', 'IPermissionPolicy',
            'IPermissionGroupProvider', 'PermissionError', 'PermissionSystem']
 
 
-class PermissionError(StandardError, TracBaseError):
-    """Insufficient permissions to perform the operation.
-
-    :since 1.0.5: the `msg` attribute is deprecated and will be removed in
-                  1.3.1. Use the `message` property instead.
-    """
+class PermissionError(TracBaseError):
+    """Insufficient permissions to perform the operation."""
 
     title = N_("Forbidden")
 
@@ -63,7 +59,6 @@ class PermissionError(StandardError, TracBaseError):
                         "permissions.", perm=self.action)
         elif msg is None:
             msg = _("Insufficient privileges to perform this operation.")
-        self.msg = msg
         super(PermissionError, self).__init__(msg)
 
     @property
@@ -71,6 +66,10 @@ class PermissionError(StandardError, TracBaseError):
         return self.args[0]
 
 
+class PermissionExistsError(TracError):
+    """Thrown when a unique key constraint is violated."""
+
+
 class IPermissionRequestor(Interface):
     """Extension point interface for components that define actions."""
 
@@ -87,26 +86,24 @@ class IPermissionRequestor(Interface):
 
 class IPermissionStore(Interface):
     """Extension point interface for components that provide storage and
-    management of permissions."""
+    management of permissions.
+    """
 
     def get_user_permissions(username):
-        """Return all permissions for the user with the specified name.
-
-        The permissions are returned as a dictionary where the key is the name
-        of the permission, and the value is either `True` for granted
-        permissions or `False` for explicitly denied permissions."""
+        """Return a list of permissions for the specified user.
+        """
 
     def get_users_with_permissions(permissions):
-        """Retrieve a list of users that have any of the specified permissions.
-
-        Users are returned as a list of usernames.
+        """Return a list of users that have any of the specified
+        permissions.
         """
 
     def get_all_permissions():
         """Return all permissions for all users.
 
         The permissions are returned as a list of (subject, action)
-        formatted tuples."""
+        formatted tuples.
+        """
 
     def grant_permission(username, action):
         """Grant a user permission to perform an action."""
@@ -116,13 +113,14 @@ class IPermissionStore(Interface):
 
 
 class IPermissionGroupProvider(Interface):
-    """Extension point interface for components that provide information about
-    user groups.
+    """Extension point interface for components that provide information
+    about user groups.
     """
 
     def get_permission_groups(username):
-        """Return a list of names of the groups that the user with the specified
-        name is a member of."""
+        """Return a list of names of the groups that the user with the
+        specified name is a member of.
+        """
 
 
 class IPermissionPolicy(Interface):
@@ -143,7 +141,7 @@ class IPermissionPolicy(Interface):
 
         :return: `True` if action is allowed, `False` if action is denied,
                  or `None` if indifferent. If `None` is returned, the next
-                 policy in the chain will be used, and so on.
+                 policy in the chain will be consulted.
 
         Note that when checking a permission on a realm resource (i.e. when
         `.id` is `None`), this usually corresponds to some preliminary check
@@ -156,8 +154,8 @@ class IPermissionPolicy(Interface):
          * `None` if the action *can not* be performed for *some* resources.
            This corresponds to situation where the policy is only interested
            in returning `False` or `None` on specific resources.
-         * `False` if the action *can not* be performed for *any* resource in
-           that realm (that's a very strong decision as that will usually
+         * `False` if the action *can not* be performed for *any* resource
+           in that realm (that's a very strong decision as that will usually
            prevent any fine-grained check to even happen).
 
         Note that performing permission checks on realm resources may seem
@@ -172,68 +170,61 @@ class DefaultPermissionStore(Component):
     This component uses the `permission` table in the database to store both
     permissions and groups.
     """
-    implements(IPermissionStore)
+    implements(IPermissionGroupProvider, IPermissionStore)
 
     group_providers = ExtensionPoint(IPermissionGroupProvider)
 
+    # IPermissionGroupProvider methods
+
+    def get_permission_groups(self, username):
+        """Return a list of names of the groups that the user with the
+        specified name is a member of.
+        """
+        return sorted(self._get_actions_and_groups({username})[1])
+
+    # IPermissionStore methods
+
     def get_user_permissions(self, username):
         """Retrieve a list of permissions for the given user.
 
         The permissions are stored in the database as (username, action)
-        records. There's simple support for groups by using lowercase names for
-        the action column: such a record represents a group and not an actual
-        permission, and declares that the user is part of that group.
+        records. There's simple support for groups by using lowercase
+        names for the action column: such a record represents a group and
+        not an actual permission, and declares that the user is part of
+        that group.
         """
-        subjects = set([username])
+        subjects = {username}
         for provider in self.group_providers:
-            subjects.update(provider.get_permission_groups(username) or [])
-
-        actions = set()
-        perms = self._all_permissions
-        while True:
-            num_users = len(subjects)
-            num_actions = len(actions)
-            for user, action in perms:
-                if user in subjects:
-                    if action.isupper() and action not in actions:
-                        actions.add(action)
-                    if not action.isupper() and action not in subjects:
-                        # action is actually the name of the permission
-                        # group here
-                        subjects.add(action)
-            if num_users == len(subjects) and num_actions == len(actions):
-                break
-        return list(actions)
+            if provider is not self:
+                subjects.update(provider.get_permission_groups(username) or [])
+        return sorted(self._get_actions_and_groups(subjects)[0])
 
     def get_users_with_permissions(self, permissions):
-        """Retrieve a list of users that have any of the specified permissions
+        """Retrieve a list of users that have any of the specified
+        permissions
 
         Users are returned as a list of usernames.
         """
-        # get_user_permissions() takes care of the magic 'authenticated' group.
-        # The optimized loop we had before didn't.  This is very inefficient,
-        # but it works.
+        # get_user_permissions() takes care of the magic 'authenticated'
+        # group. The optimized loop we had before didn't. This is very
+        # inefficient, but it works.
         result = set()
-        users = set([u[0] for u in self.env.get_known_users()])
+        users = {u[0] for u in self.env.get_known_users()}
         for user in users:
-            userperms = self.get_user_permissions(user)
+            user_perms = self.get_user_permissions(user)
             for group in permissions:
-                if group in userperms:
+                if group in user_perms:
                     result.add(user)
-        return list(result)
+        return sorted(result)
 
     def get_all_permissions(self):
         """Return all permissions for all users.
 
         The permissions are returned as a list of (subject, action)
-        formatted tuples."""
+        formatted tuples.
+        """
         return self._all_permissions
 
-    @cached
-    def _all_permissions(self):
-        return [(username, action) for username, action in
-                self.env.db_query("SELECT username, action FROM permission")]
-
     def grant_permission(self, username, action):
         """Grants a user the permission to perform the specified action."""
         self.env.db_transaction("INSERT INTO permission VALUES (%s, %s)",
@@ -253,10 +244,38 @@ class DefaultPermissionStore(Component):
         # Invalidate cached property
         del self._all_permissions
 
+    @cached
+    def _all_permissions(self):
+        return sorted(self.env.db_query("""
+                          SELECT username, action FROM permission
+                          """))
+
+    def _get_actions_and_groups(self, subjects):
+        """Get actions and groups for `subjects`, an iterable of username
+        and groups that username is a member of.
+        """
+        actions = set()
+        groups = set()
+        perms = self._all_permissions
+        while True:
+            num_users = len(subjects)
+            num_actions = len(actions)
+            for user, action in perms:
+                if user in subjects:
+                    if action.isupper():
+                        actions.add(action)
+                    if not action.isupper():  # permission group
+                        subjects.add(action)
+                        groups.add(action)
+            if num_users == len(subjects) and num_actions == len(actions):
+                break
+        return actions, groups
+
 
 class DefaultPermissionGroupProvider(Component):
-    """Permission group provider providing the basic builtin permission groups
-    'anonymous' and 'authenticated'."""
+    """Permission group provider providing the basic builtin permission
+    groups 'anonymous' and 'authenticated'.
+    """
 
     required = True
 
@@ -292,7 +311,8 @@ class DefaultPermissionPolicy(Component):
             self.permission_cache = {}
             self.last_reap = time_now()
 
-        timestamp, permissions = self.permission_cache.get(username, (0, None))
+        timestamp, permissions = \
+            self.permission_cache.get(username, (0, None))
 
         # Cache hit?
         if now - timestamp > self.CACHE_EXPIRY:
@@ -312,15 +332,17 @@ class PermissionSystem(Component):
     implements(IPermissionRequestor)
 
     requestors = ExtensionPoint(IPermissionRequestor)
+    group_providers = ExtensionPoint(IPermissionGroupProvider)
 
     store = ExtensionOption('trac', 'permission_store', IPermissionStore,
                             'DefaultPermissionStore',
-        """Name of the component implementing `IPermissionStore`, which is used
-        for managing user and group permissions.""")
+        """Name of the component implementing `IPermissionStore`, which is
+        used for managing user and group permissions.""")
 
     policies = OrderedExtensionsOption('trac', 'permission_policies',
         IPermissionPolicy,
-        'ReadonlyWikiPolicy, DefaultPermissionPolicy, LegacyAttachmentPolicy',
+        'DefaultWikiPolicy, DefaultTicketPolicy, DefaultPermissionPolicy, '
+        'LegacyAttachmentPolicy',
         False,
         """List of components implementing `IPermissionPolicy`, in the order
         in which they will be applied. These components manage fine-grained
@@ -339,7 +361,13 @@ class PermissionSystem(Component):
 
     def grant_permission(self, username, action):
         """Grant the user with the given name permission to perform to
-        specified action."""
+        specified action.
+
+        :raises PermissionExistsError: if user already has the permission
+                                       or is a member of the group.
+
+        :since 1.3.1: raises PermissionExistsError rather than IntegrityError
+        """
         if action.isupper() and action not in self.get_actions():
             raise TracError(_('%(name)s is not a valid action.', name=action))
         elif not action.isupper() and action.upper() in self.get_actions():
@@ -347,7 +375,17 @@ class PermissionSystem(Component):
                               "action by casing only, which is not allowed.",
                               name=action))
 
-        self.store.grant_permission(username, action)
+        try:
+            self.store.grant_permission(username, action)
+        except self.env.db_exc.IntegrityError:
+            if action in self.get_actions():
+                raise PermissionExistsError(
+                    _("The user %(user)s already has permission %(action)s.",
+                      user=username, action=action))
+            else:
+                raise PermissionExistsError(
+                    _("The user %(user)s is already in the group %(group)s.",
+                      user=username, group=action))
 
     def revoke_permission(self, username, action):
         """Revokes the permission of the specified user to perform an
@@ -385,7 +423,7 @@ class PermissionSystem(Component):
                     actions.add(action[0])
                 else:
                     actions.add(action)
-        return list(actions)
+        return sorted(actions)
 
     def get_groups_dict(self):
         """Get all groups as a `dict`.
@@ -397,8 +435,8 @@ class PermissionSystem(Component):
         groups = sorted((p for p in self.get_all_permissions()
                            if not p[1].isupper()), key=lambda p: p[1])
 
-        return dict((k, sorted(i[0] for i in list(g)))
-                    for k, g in groupby(groups, key=lambda p: p[1]))
+        return {k: sorted(i[0] for i in list(g))
+                for k, g in groupby(groups, key=lambda p: p[1])}
 
     def get_users_dict(self):
         """Get all users as a `dict`.
@@ -411,36 +449,58 @@ class PermissionSystem(Component):
         perms = sorted((p for p in self.get_all_permissions()
                           if p[1].isupper()), key=lambda p: p[0])
 
-        return dict((k, sorted(i[1] for i in list(g)))
-                    for k, g in groupby(perms, key=lambda p: p[0]))
+        return {k: sorted(i[1] for i in list(g))
+                for k, g in groupby(perms, key=lambda p: p[0])}
 
-    def get_user_permissions(self, username=None):
+    def get_user_permissions(self, username=None, undefined=False,
+                             expand_meta=True):
         """Return the permissions of the specified user.
 
-        The return value is a dictionary containing all the actions granted to
-        the user mapped to `True`. If an action is missing as a key, or has
-        `False` as a value, permission is denied."""
+        The return value is a dictionary containing all the actions
+        granted to the user mapped to `True`.
+
+        :param undefined: if `True`, include actions that are not defined
+            in any of the `IPermissionRequestor` implementations.
+        :param expand_meta: if `True`, expand meta permissions.
+
+        :since 1.3.1: added the `undefined` parameter.
+        :since 1.3.3: added the `expand_meta` parameter.
+        """
         if not username:
             # Return all permissions available in the system
             return dict.fromkeys(self.get_actions(), True)
 
         # Return all permissions that the given user has
         actions = self.get_actions_dict()
-        permissions = {}
-        def expand_meta(action):
-            if action not in permissions:
-                permissions[action] = True
-                for a in actions.get(action, ()):
-                    expand_meta(a)
-        for perm in self.store.get_user_permissions(username) or []:
-            expand_meta(perm)
-        return permissions
+        user_permissions = self.store.get_user_permissions(username) or []
+        if expand_meta:
+            return {p: True for p in self.expand_actions(user_permissions)
+                            if undefined or p in actions}
+        else:
+            return {p: True for p in user_permissions
+                            if undefined or p in actions}
+
+    def get_permission_groups(self, username):
+        """Return a sorted list of groups that `username` belongs to.
+
+        Groups are recursively expanded such that if `username` is a
+        member of `group1` and `group1` is a member of `group2`, both
+        `group1` and `group2` will be returned.
+
+        :since: 1.3.3
+        """
+        user_groups = set()
+        for provider in self.group_providers:
+            user_groups.update(provider.get_permission_groups(username) or [])
+
+        return sorted(user_groups)
 
     def get_all_permissions(self):
         """Return all permissions for all users.
 
         The permissions are returned as a list of (subject, action)
-        formatted tuples."""
+        formatted tuples.
+        """
         return self.store.get_all_permissions() or []
 
     def get_users_with_permission(self, permission):
@@ -485,12 +545,13 @@ class PermissionSystem(Component):
                     expand_action(a)
         for a in actions:
             expand_action(a)
-        return expanded_actions
+        return sorted(expanded_actions)
 
     def check_permission(self, action, username=None, resource=None,
                          perm=None):
-        """Return True if permission to perform action for the given resource
-        is allowed."""
+        """Return True if permission to perform action for the given
+        resource is allowed.
+        """
         if username is None:
             username = 'anonymous'
         if resource and resource.realm is None:
@@ -629,29 +690,29 @@ class PermissionAdmin(Component):
 
     def get_admin_commands(self):
         yield ('permission list', '[user]',
-               'List permission rules',
+               "List permission rules",
                self._complete_list, self._do_list)
         yield ('permission add', '<user> <action> [action] [...]',
-               'Add a new permission rule',
+               "Add a new permission rule",
                self._complete_add, self._do_add)
         yield ('permission remove', '<user> <action> [action] [...]',
-               'Remove a permission rule',
+               "Remove a permission rule",
                self._complete_remove, self._do_remove)
         yield ('permission export', '[file]',
-               'Export permission rules to a file or stdout as CSV',
+               "Export permission rules to a file or stdout as CSV",
                self._complete_import_export, self._do_export)
         yield ('permission import', '[file]',
-               'Import permission rules from a file or stdin as CSV',
+               "Import permission rules from a file or stdin as CSV",
                self._complete_import_export, self._do_import)
 
     def get_user_list(self):
-        return set(user for (user, action) in
-                   PermissionSystem(self.env).get_all_permissions())
+        return {user for (user, action)
+                     in PermissionSystem(self.env).get_all_permissions()}
 
     def get_user_perms(self, user):
-        return [action for (subject, action) in
-                PermissionSystem(self.env).get_all_permissions()
-                if subject == user]
+        return [action for (subject, action)
+                       in PermissionSystem(self.env).get_all_permissions()
+                       if subject == user]
 
     def _complete_list(self, args):
         if len(args) == 1:
@@ -678,7 +739,7 @@ class PermissionAdmin(Component):
         permsys = PermissionSystem(self.env)
         if user:
             rows = []
-            perms = permsys.get_user_permissions(user)
+            perms = permsys.get_user_permissions(user, undefined=True)
             for action in perms:
                 if perms[action]:
                     rows.append((user, action))
@@ -698,17 +759,32 @@ class PermissionAdmin(Component):
     def _do_add(self, user, *actions):
         permsys = PermissionSystem(self.env)
         if user.isupper():
-            raise AdminCommandError(_('All upper-cased tokens are reserved '
-                                      'for permission names'))
-        for action in actions:
+            raise AdminCommandError(_("All upper-cased tokens are reserved "
+                                      "for permission names"))
+
+        def grant_actions_atomically(actions):
+            action = None
             try:
-                permsys.grant_permission(user, action)
-            except self.env.db_exc.IntegrityError:
-                printout(_("The user %(user)s already has permission "
-                           "%(action)s.", user=user, action=action))
+                with self.env.db_transaction:
+                    for action in actions:
+                        permsys.grant_permission(user, action)
+            except PermissionExistsError as e:
+                printout(e)
+                return action
             except TracError as e:
                 raise AdminCommandError(e)
 
+        # An exception rolls back the atomic transaction so it's
+        # necessary to retry the transaction after removing the
+        # failed action from the list.
+        actions_to_grant = list(actions)
+        while actions_to_grant:
+            action_that_failed = grant_actions_atomically(actions_to_grant)
+            if action_that_failed is None:
+                break
+            else:
+                actions_to_grant.remove(action_that_failed)
+
     def _do_remove(self, user, *actions):
         permsys = PermissionSystem(self.env)
         rows = permsys.get_all_permissions()
@@ -752,10 +828,10 @@ class PermissionAdmin(Component):
 
     def _do_import(self, filename=None):
         permsys = PermissionSystem(self.env)
+        linesep = os.linesep if filename else '\n'
         try:
             with file_or_std(filename, 'rb') as f:
                 encoding = stream_encoding(f)
-                linesep = os.linesep if filename else '\n'
                 reader = csv.reader(f, lineterminator=linesep)
                 for row in reader:
                     if len(row) < 2:
@@ -769,8 +845,9 @@ class PermissionAdmin(Component):
                     if user.isupper():
                         raise AdminCommandError(
                             _("Invalid user %(user)s on line %(line)d: All "
-                              "upper-cased tokens are reserved for permission "
-                              "names.", user=user, line=reader.line_num))
+                              "upper-cased tokens are reserved for "
+                              "permission names.",
+                              user=user, line=reader.line_num))
                     old_actions = self.get_user_perms(user)
                     for action in set(actions) - set(old_actions):
                         permsys.grant_permission(user, action)
