@@ -1,34 +1,35 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2018 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004-2005 Daniel Lundin <daniel@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin <daniel@edgewall.com>
 
+import math
 import pkg_resources
 import re
 
-from genshi.builder import tag
-
 from trac.core import *
 from trac.prefs.api import IPreferencePanelProvider
-from trac.util import lazy
+from trac.util import as_float, lazy
 from trac.util.datefmt import all_timezones, get_timezone, localtz
+from trac.util.html import tag
 from trac.util.translation import _, Locale, deactivate,\
                                   get_available_locales, make_activable
 from trac.web.api import HTTPNotFound, IRequestHandler, \
                          is_valid_default_handler
 from trac.web.chrome import Chrome, INavigationContributor, \
-                            ITemplateProvider, add_notice, add_stylesheet
+                            ITemplateProvider, add_notice, add_stylesheet, \
+                            add_warning
 
 
 class PreferencesModule(Component):
@@ -90,26 +91,28 @@ class PreferencesModule(Component):
             raise HTTPNotFound(_("Unknown preference panel '%(panel)s'",
                                  panel=panel_id))
 
-        session_data = {
-            'session': req.session,
-            'settings': {'session': req.session,  # Compat: remove in 1.3.1
-                         'session_id': req.session.sid},
-        }
+        session_data = {'session': req.session}
 
         # Render child preference panels.
         chrome = Chrome(self.env)
         children = []
         if child_panels.get(panel_id):
             for name, label in child_panels[panel_id]:
-                ctemplate, cdata = \
-                    providers[name].render_preference_panel(req, name)
+                resp = providers[name].render_preference_panel(req, name)
+                ctemplate, cdata = resp[:2]
                 cdata.update(session_data)
-                rendered = chrome.render_template(req, ctemplate, cdata,
-                                                  fragment=True)
+                if len(resp) == 2:
+                    rendered = chrome.render_fragment(req, ctemplate, cdata)
+                else:
+                    # Backward compatibility with Genshi preference panels
+                    # TODO (1.5.1) remove
+                    rendered = chrome.render_template(req, ctemplate, cdata,
+                                                      None, fragment=True)
                 children.append((name, label, rendered))
 
-        template, data = \
-            chosen_provider.render_preference_panel(req, panel_id)
+        resp = chosen_provider.render_preference_panel(req, panel_id)
+        data = resp[1]
+
         data.update(session_data)
         data.update({
             'active_panel': panel_id,
@@ -118,7 +121,7 @@ class PreferencesModule(Component):
         })
 
         add_stylesheet(req, 'common/css/prefs.css')
-        return template, data, None
+        return resp
 
     # ITemplateProvider methods
 
@@ -159,7 +162,7 @@ class AdvancedPreferencePanel(Component):
     # IPreferencePanelProvider methods
 
     def get_preference_panels(self, req):
-        if not req.authname or req.authname == 'anonymous':
+        if not req.is_authenticated:
             yield 'advanced', _("Advanced")
 
     def render_preference_panel(self, req, panel):
@@ -171,7 +174,7 @@ class AdvancedPreferencePanel(Component):
         return 'prefs_advanced.html', {'session_id': req.session.sid}
 
     def _do_load(self, req):
-        if req.authname == 'anonymous':
+        if not req.is_authenticated:
             oldsid = req.args.get('loadsid')
             if oldsid:
                 req.session.get_session(oldsid)
@@ -195,23 +198,6 @@ class GeneralPreferencePanel(Component):
         return 'prefs_general.html', {}
 
 
-class KeyBindingsPreferencePanel(Component):
-
-    implements(IPreferencePanelProvider)
-
-    _form_fields = ('accesskeys',)
-
-    # IPreferencePanelProvider methods
-
-    def get_preference_panels(self, req):
-        yield 'keybindings', _("Keyboard Shortcuts")
-
-    def render_preference_panel(self, req, panel):
-        if req.method == 'POST':
-            _do_save(req, panel, self._form_fields)
-        return 'prefs_keybindings.html', {}
-
-
 class LocalizationPreferencePanel(Component):
 
     implements(IPreferencePanelProvider)
@@ -260,7 +246,9 @@ class UserInterfacePreferencePanel(Component):
     implements(IPreferencePanelProvider)
 
     _request_handlers = ExtensionPoint(IRequestHandler)
-    _form_fields = ('default_handler', 'ui.hide_help', 'ui.use_symbols')
+
+    _form_fields = ('accesskeys', 'default_handler','ui.auto_preview_timeout',
+                    'ui.hide_help', 'ui.use_symbols', 'wiki_fullwidth')
 
     # IPreferencePanelProvider methods
 
@@ -271,9 +259,11 @@ class UserInterfacePreferencePanel(Component):
         if req.method == 'POST':
             _do_save(req, panel, self._form_fields)
 
+        auto_preview_timeout = self.config.get('trac', 'auto_preview_timeout')
         data = {
             'project_default_handler': self._project_default_handler,
             'valid_default_handlers': self._valid_default_handlers,
+            'default_auto_preview_timeout': auto_preview_timeout,
         }
         return 'prefs_userinterface.html', data
 
@@ -294,9 +284,16 @@ def _do_save(req, panel, form_fields):
     for field in form_fields:
         val = req.args.get(field, '').strip()
         if val:
+            if field == 'ui.auto_preview_timeout':
+                fval = as_float(val, default=None)
+                if fval is None or math.isinf(fval) or math.isnan(fval) \
+                        or fval < 0:
+                    add_warning(req, _("Discarded invalid value \"%(val)s\" "
+                                       "for auto preview timeout.", val=val))
+                    continue
             if field == 'tz' and 'tz' in req.session and \
                     val not in all_timezones:
-                del req.session['tz']
+                del req.session[field]
             elif field == 'newsid':
                 req.session.change_sid(val)
             else:
