@@ -137,12 +137,12 @@ class TicketModule(Component):
     # IContentConverter methods
 
     def get_supported_conversions(self):
+        yield ('rss', _("RSS Feed"), 'xml',
+               'trac.ticket.Ticket', 'application/rss+xml', 8)
         yield ('csv', _("Comma-delimited Text"), 'csv',
                'trac.ticket.Ticket', 'text/csv', 8)
         yield ('tab', _("Tab-delimited Text"), 'tsv',
                'trac.ticket.Ticket', 'text/tab-separated-values', 8)
-        yield ('rss', _("RSS Feed"), 'xml',
-               'trac.ticket.Ticket', 'application/rss+xml', 8)
 
     def convert_content(self, req, mimetype, ticket, key):
         if key == 'csv':
@@ -770,8 +770,13 @@ class TicketModule(Component):
                     break
 
         # Data need for Javascript-specific logic
-        old_values = {name: ticket[name]
-                      for name in [field['name'] for field in ticket.fields]}
+        old_values = {}
+        for field in ticket.fields:
+            name = field['name']
+            value = ticket._old.get(name, ticket[name])
+            if name in ticket.time_fields and isinstance(value, datetime):
+                value = format_datetime(value, 'iso8601', utc)
+            old_values[name] = value
         old_values['id'] = ticket.id
         add_script_data(req, {'comments_prefs': self._get_prefs(req),
                               'old_values': old_values,
