@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -8,24 +8,30 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Matthew Good <trac@matt-good.net>
 
-from cStringIO import StringIO
 import csv
 import errno
 import functools
 import hashlib
+import importlib
 import inspect
-from itertools import izip, tee
+import io
+try:
+    from itertools import izip, tee
+except ImportError:
+    import sys
+    sys.exit("Python 3 not supported... yet! (#12130)")
 import locale
 import os
+import posixpath
 from pkg_resources import find_distributions
 import random
 import re
@@ -34,12 +40,13 @@ import sys
 import string
 import struct
 import tempfile
+import unicodedata
+import zipfile
 from urllib import quote, unquote, urlencode
 
-from trac.util.compat import any, md5, sha1, sorted  # Remove in 1.3.1
 from trac.util.datefmt import time_now, to_datetime, to_timestamp, utc
-from trac.util.text import exception_to_unicode, to_unicode, \
-                           getpreferredencoding
+from trac.util.text import exception_to_unicode, getpreferredencoding, \
+                           stripws, to_unicode
 
 
 def get_reporter_id(req, arg_name=None):
@@ -53,7 +60,7 @@ def get_reporter_id(req, arg_name=None):
     :param arg_name: if given, a `Request` argument which may contain
       the id for non-authentified users
     """
-    if req.authname != 'anonymous':
+    if req.is_authenticated:
         return req.authname
     if arg_name:
         r = req.args.get(arg_name)
@@ -97,6 +104,55 @@ except NameError:
     class WindowsError(OSError):
         """Dummy exception replacing WindowsError on non-Windows platforms"""
 
+def native_path(path):
+    """Converts a Windows-style or POSIX-style path to the native style.
+
+    i.e. on Windows, convert POSIX path to Windows path, and in a
+    POSIX system, convert Windows path to POSIX path.
+
+    :param path: the input path
+    :return: the path converted to native style
+    """
+    if path:
+        if os.name == 'posix':
+            if (len(path) > 1 and path[1] == ':') or '\\' in path:
+                path = path.replace('\\', '/')
+                if path[1] == ':':
+                    path = '/' + path[0] + path[2:]
+        elif os.name == 'nt':
+            if path[0] == '/': # abs path
+                if len(path) == 2 or len(path) > 2 and path[2] == '/':
+                    # interpret 1-letter toplevel as volume name
+                    path = path[1] + ':' + path[2:]
+                else:
+                    # assume we're talking about C:
+                    path = 'C:\\' + path[1:]
+            path = path.replace('/', '\\')
+    return path
+
+
+_control_codes_re = re.compile(
+    '[' +
+    ''.join(filter(lambda c: unicodedata.category(c) == 'Cc',
+                   map(unichr, xrange(0x10000)))) +
+    ']')
+
+def normalize_filename(filepath):
+    # We try to normalize the filename to unicode NFC if we can.
+    # Files from OS X might be in NFD.
+    if not isinstance(filepath, unicode):
+        filepath = unicode(filepath, 'utf-8')
+    filepath = unicodedata.normalize('NFC', filepath)
+    # Replace control codes with spaces, e.g. NUL, LF, DEL, U+009F
+    filepath = _control_codes_re.sub(' ', filepath)
+    # Replace backslashes with slashes if filename is Windows full path
+    if filepath.startswith('\\') or re.match(r'[A-Za-z]:\\', filepath):
+        filepath = filepath.replace('\\', '/')
+    # We want basename to be delimited by only slashes on all platforms
+    filename = posixpath.basename(filepath)
+    filename = stripws(filename)
+    return filename
+
 
 can_rename_open_file = False
 if os.name == 'nt':
@@ -249,13 +305,13 @@ def create_file(path, data='', mode='w'):
 def create_unique_file(path):
     """Create a new file. An index is added if the path exists"""
     parts = os.path.splitext(path)
+    flags = os.O_CREAT + os.O_WRONLY + os.O_EXCL
+    if hasattr(os, 'O_BINARY'):
+        flags += os.O_BINARY
     idx = 1
     while 1:
         try:
-            flags = os.O_CREAT + os.O_WRONLY + os.O_EXCL
-            if hasattr(os, 'O_BINARY'):
-                flags += os.O_BINARY
-            return path, os.fdopen(os.open(path, flags, 0666), 'w')
+            return path, os.fdopen(os.open(path, flags, 0o666), 'w')
         except OSError as e:
             if e.errno != errno.EEXIST:
                 raise
@@ -289,6 +345,18 @@ else:
                 raise
 
 
+def backup_config_file(env, suffix):
+    try:
+        backup, f = create_unique_file(env.config.filename + suffix)
+        f.close()
+        shutil.copyfile(env.config.filename, backup)
+    except IOError as e:
+        env.log.warning("Couldn't save backup of configuration file (%s)",
+                        exception_to_unicode(e))
+    else:
+        env.log.info("Saved backup of configuration file in %s", backup)
+
+
 def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=False,
                    comment=None):
     """Create a instance of `ZipInfo`.
@@ -300,8 +368,7 @@ def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=Fa
     :param symlink: if `True`, the entry is a symbolic link
     :param comment: comment of the entry
     """
-    from zipfile import ZipInfo, ZIP_DEFLATED, ZIP_STORED
-    zipinfo = ZipInfo()
+    zipinfo = zipfile.ZipInfo()
 
     # The general purpose bit flag 11 is used to denote
     # UTF-8 encoding for path and comment. Only set it for
@@ -333,17 +400,17 @@ def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=Fa
     if dir:
         if not zipinfo.filename.endswith('/'):
             zipinfo.filename += '/'
-        zipinfo.compress_type = ZIP_STORED
-        zipinfo.external_attr = 040755 << 16L        # permissions drwxr-xr-x
+        zipinfo.compress_type = zipfile.ZIP_STORED
+        zipinfo.external_attr = 0o40755 << 16        # permissions drwxr-xr-x
         zipinfo.external_attr |= 0x10                # MS-DOS directory flag
     else:
-        zipinfo.compress_type = ZIP_DEFLATED
-        zipinfo.external_attr = 0644 << 16L          # permissions -r-wr--r--
+        zipinfo.compress_type = zipfile.ZIP_DEFLATED
+        zipinfo.external_attr = 0o644 << 16          # permissions -r-wr--r--
         if executable:
-            zipinfo.external_attr |= 0755 << 16L     # -rwxr-xr-x
+            zipinfo.external_attr |= 0o755 << 16     # -rwxr-xr-x
         if symlink:
-            zipinfo.compress_type = ZIP_STORED
-            zipinfo.external_attr |= 0120000 << 16L  # symlink file type
+            zipinfo.compress_type = zipfile.ZIP_STORED
+            zipinfo.external_attr |= 0o120000 << 16  # symlink file type
 
     if comment:
         zipinfo.comment = comment.encode('utf-8')
@@ -351,6 +418,20 @@ def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=Fa
     return zipinfo
 
 
+def extract_zipfile(srcfile, destdir):
+    with zipfile.ZipFile(srcfile) as zip:
+        for entry in zip.namelist():
+            if entry.endswith('/'):  # is a directory
+                continue
+            names = entry.split('/')
+            content = zip.read(entry)
+            filename = os.path.join(destdir, *names)
+            dirname = os.path.dirname(filename)
+            if not os.path.isdir(dirname):
+                os.makedirs(dirname)
+            create_file(filename, content, 'wb')
+
+
 class NaivePopen(object):
     """This is a deadlock-safe version of popen that returns an object with
     errorlevel, out (a string) and err (a string).
@@ -369,9 +450,8 @@ class NaivePopen(object):
         command = '( %s ) > %s' % (command, outfile)
         if input is not None:
             infile = tempfile.mktemp()
-            tmp = open(infile, 'w')
-            tmp.write(input)
-            tmp.close()
+            with open(infile, 'w') as tmp:
+                tmp.write(input)
             command = command + ' <' + infile
         if capturestderr:
             errfile = tempfile.mktemp()
@@ -548,7 +628,7 @@ def arity(f):
 def get_last_traceback():
     """Retrieve the last traceback as an `unicode` string."""
     import traceback
-    tb = StringIO()
+    tb = io.BytesIO()
     traceback.print_exc(file=tb)
     return to_unicode(tb.getvalue())
 
@@ -566,15 +646,11 @@ def get_lines_from_file(filename, lineno, context=0, globals=None):
     lines = []
     match = _egg_path_re.match(filename)
     if match:
-        import zipfile
         for path in sys.path:
             try:
-                zip = zipfile.ZipFile(path, 'r')
-                try:
+                with zipfile.ZipFile(path, 'r') as zip:
                     lines = zip.read(match.group(1)).splitlines()
                     break
-                finally:
-                    zip.close()
             except Exception:
                 pass
 
@@ -639,7 +715,7 @@ def safe__import__(module_name):
     """
     already_imported = sys.modules.copy()
     try:
-        return __import__(module_name, globals(), locals(), [])
+        return importlib.import_module(module_name)
     except Exception as e:
         for modname in sys.modules.copy():
             if modname not in already_imported:
@@ -685,8 +761,7 @@ def import_namespace(globals_dict, module_name):
     another module into the global namespace of the stub, usually for
     backward compatibility.
     """
-    __import__(module_name)
-    module = sys.modules[module_name]
+    module = importlib.import_module(module_name)
     globals_dict.update(item for item in module.__dict__.iteritems()
                         if item[0] not in _dont_import)
     globals_dict.pop('import_namespace', None)
@@ -727,7 +802,7 @@ def get_sources(path):
                            if any(src.startswith(top) for top in toplevels))
             continue
         if dist.has_metadata('RECORD'):  # *.dist-info/RECORD
-            reader = csv.reader(StringIO(dist.get_metadata('RECORD')))
+            reader = csv.reader(io.BytesIO(dist.get_metadata('RECORD')))
             sources.update((row[0], dist)
                            for row in reader if any(row[0].startswith(top)
                                                     for top in toplevels))
@@ -767,7 +842,7 @@ def get_pkginfo(dist):
                 return any(resource_name == os.path.normpath(name)
                            for name in dist.get_metadata_lines('SOURCES.txt'))
             if dist.has_metadata('RECORD'):  # *.dist-info/RECORD
-                reader = csv.reader(StringIO(dist.get_metadata('RECORD')))
+                reader = csv.reader(io.BytesIO(dist.get_metadata('RECORD')))
                 return any(resource_name == row[0] for row in reader)
             if dist.has_metadata('PKG-INFO'):
                 try:
@@ -798,8 +873,9 @@ def get_pkginfo(dist):
         else:
             return {}
 
-    attrs = ('author', 'author-email', 'license', 'home-page', 'summary',
-             'name', 'description', 'version')
+    attrs = ('author', 'author-email', 'maintainer', 'maintainer-email',
+             'license', 'home-page', 'summary', 'name', 'description',
+             'version')
     info = {}
     def normalize(attr):
         return attr.lower().replace('-', '_')
@@ -821,6 +897,32 @@ def get_pkginfo(dist):
     return info
 
 
+def get_module_metadata(module):
+    """Get a dictionary containing metadata for a module."""
+    info = {}
+    for k in ('author', 'author_email', 'maintainer',
+              'maintainer_email', 'home_page', 'url', 'license',
+              'summary', 'trac'):
+        v = getattr(module, k, '')
+        if v and isinstance(v, basestring):
+            if k in ('home_page', 'url'):
+                k = 'home_page'
+                v = v.replace('$', '').replace('URL: ', '').strip()
+            else:
+                v = to_unicode(v)
+            info[k] = v
+    # retrieve plugin version info
+    version = (getattr(module, 'version', '') or
+               getattr(module, 'revision', ''))
+    # special handling for "$Rev$" strings
+    if version != '$Rev$':
+        info['version'] = \
+            version.replace('$', '').replace('Rev: ', 'r').strip()
+    else:  # keyword hasn't been expanded
+        info['version'] = ''
+    return info
+
+
 def warn_setuptools_issue(out=None):
     if not out:
         out = sys.stderr
@@ -862,7 +964,7 @@ def hex_entropy(digits=32):
 def salt(length=2):
     """Returns a string of `length` random letters and numbers."""
     return ''.join(random.choice(string.ascii_letters + string.digits + '/.')
-                   for x in range(length))
+                   for x in xrange(length))
 
 
 # Original license for md5crypt:
@@ -886,7 +988,7 @@ def md5crypt(password, salt, magic='$1$'):
 
     # /* Then just as many characters of the MD5(pw,salt,pw) */
     mixin = hashlib.md5(password + salt + password).digest()
-    for i in range(0, len(password)):
+    for i in xrange(len(password)):
         m.update(mixin[i % 16])
 
     # /* Then something really weird... */
@@ -902,7 +1004,7 @@ def md5crypt(password, salt, magic='$1$'):
     final = m.digest()
 
     # /* and now, just to make sure things don't run too fast */
-    for i in range(1000):
+    for i in xrange(1000):
         m2 = hashlib.md5()
         if i & 1:
             m2.update(password)
@@ -929,12 +1031,12 @@ def md5crypt(password, salt, magic='$1$'):
     rearranged = ''
     for a, b, c in ((0, 6, 12), (1, 7, 13), (2, 8, 14), (3, 9, 15), (4, 10, 5)):
         v = ord(final[a]) << 16 | ord(final[b]) << 8 | ord(final[c])
-        for i in range(4):
+        for i in xrange(4):
             rearranged += itoa64[v & 0x3f]
             v >>= 6
 
     v = ord(final[11])
-    for i in range(2):
+    for i in xrange(2):
         rearranged += itoa64[v & 0x3f]
         v >>= 6
 
@@ -957,7 +1059,7 @@ class Ranges(object):
     True
     >>> 16 in x
     False
-    >>> [i for i in range(20) if i in x]
+    >>> [i for i in xrange(20) if i in x]
     [1, 2, 9, 10, 11, 12, 13, 14, 15]
 
     Also supports iteration, which makes that last example a bit simpler:
@@ -1016,7 +1118,7 @@ class Ranges(object):
 
     """
 
-    RE_STR = ur'[0-9]+(?:[-:][0-9]+)?(?:,\u200b?[0-9]+(?:[-:][0-9]+)?)*'
+    RE_STR = u'[0-9]+(?:[-:][0-9]+)?(?:,\u200b?[0-9]+(?:[-:][0-9]+)?)*'
 
     def __init__(self, r=None, reorder=False):
         self.pairs = []
@@ -1073,7 +1175,7 @@ class Ranges(object):
         return chain(*[xrange(a, b+1) for a, b in self.pairs])
         """
         for a, b in self.pairs:
-            for i in range(a, b+1):
+            for i in xrange(a, b+1):
                 yield i
 
     def __contains__(self, x):
@@ -1240,20 +1342,36 @@ def partition(iterable, order=None):
     return [result[key] for key in order]
 
 
-def as_int(s, default, min=None, max=None):
-    """Convert s to an int and limit it to the given range, or return default
-    if unsuccessful."""
+def _as_numeric(numeric_type, s, default, min, max):
     try:
-        value = int(s)
+        value = numeric_type(s)
     except (TypeError, ValueError):
         return default
     if min is not None and value < min:
-        value = min
+        value = numeric_type(min)  # cast to ensure proper type
     if max is not None and value > max:
-        value = max
+        value = numeric_type(max)  # cast to ensure proper type
     return value
 
 
+def as_int(s, default=None, min=None, max=None):
+    """Convert s to an int and limit it to the given range, or return default
+    if unsuccessful.
+
+    :since 1.3.6: the default value of the `default` argument is `None`
+    """
+    return _as_numeric(int, s, default, min, max)
+
+
+def as_float(s, default=None, min=None, max=None):
+    """Convert s to a float and limit it to the given range, or return default
+    if unsuccessful.
+
+    :since: 1.3.6
+    """
+    return _as_numeric(float, s, default, min, max)
+
+
 def as_bool(value, default=False):
     """Convert the given value to a `bool`.
 
@@ -1293,16 +1411,20 @@ def to_list(splittable, sep=','):
     ['1', '2', '3', '4']
     >>> to_list('1;2; 3;4 ', sep=';')
     ['1', '2', '3', '4']
+    >>> to_list('1,2;3 4 ', sep='[,;\s]+')
+    ['1', '2', '3', '4']
     >>> to_list('')
     []
     >>> to_list(None)
     []
     >>> to_list([])
     []
+
+    :since 1.3.6: the `sep` argument may be a regular expression
     """
     if not splittable:
         return []
-    split = [x.strip() for x in splittable.split(sep)]
+    split = [x.strip() for x in re.split(sep, splittable)]
     return [item for item in split if item]
 
 
