@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Matthew Good <trac@matt-good.net>
@@ -27,6 +27,8 @@ import textwrap
 from urllib import quote, quote_plus, unquote
 from unicodedata import east_asian_width
 
+import jinja2
+
 CRLF = '\r\n'
 
 class Empty(unicode):
@@ -38,6 +40,52 @@ empty = Empty()
 del Empty # shouldn't be used outside of Trac core
 
 
+# -- Jinja2
+
+def jinja2env(**kwargs):
+    """Creates a Jinja2 ``Environment`` configured with Trac conventions.
+
+    All default parameters can optionally be overriden. The ``loader``
+    parameter is not set by default, so unless it is set by the
+    caller, only inline templates can be created from the environment.
+
+    :rtype: `jinja.Environment`
+
+    """
+    exts = ('html', 'rss', 'xml')
+    def filterout_none(v):
+        return '' if v is None else v
+    def autoescape_extensions(template):
+        return template and template.rsplit('.', 1)[1] in exts
+    defaults = dict(
+        variable_start_string='${',
+        variable_end_string='}',
+        line_statement_prefix='#',
+        line_comment_prefix='##',
+        trim_blocks=True,
+        lstrip_blocks=True,
+        extensions=['jinja2.ext.do', 'jinja2.ext.i18n', 'jinja2.ext.with_'],
+        finalize=filterout_none,
+        autoescape=autoescape_extensions,
+    )
+    defaults.update(kwargs)
+    jenv = jinja2.Environment(**defaults)
+    jenv.globals.update(
+        len=len,
+    )
+    return jenv
+
+def jinja2template(template, text=False):
+    """Creates a Jinja2 ``Template`` from inlined source.
+
+    :param template: the template content
+    :param text: if set to `False`, the result of the variable
+                 expansion will be XML/HTML escaped
+
+    """
+    return jinja2env(autoescape=not text).from_string(template)
+
+
 # -- Unicode
 
 def to_unicode(text, charset=None):
@@ -58,7 +106,25 @@ def to_unicode(text, charset=None):
         except UnicodeDecodeError:
             return unicode(text, 'latin1')
     elif isinstance(text, Exception):
-        if os.name == 'nt' and isinstance(text, (OSError, IOError)):
+        if os.name == 'nt' and isinstance(text, EnvironmentError):
+            strerror = text.strerror
+            filename = text.filename
+            if isinstance(strerror, basestring) and \
+                    isinstance(filename, basestring):
+                try:
+                    if not isinstance(strerror, unicode):
+                        strerror = unicode(strerror, 'mbcs')
+                    if not isinstance(filename, unicode):
+                        filename = unicode(filename, 'mbcs')
+                except UnicodeError:
+                    pass
+                else:
+                    if isinstance(text, WindowsError):
+                        return u"[Error %s] %s: '%s'" % (text.winerror,
+                                                         strerror, filename)
+                    else:
+                        return u"[Errno %s] %s: '%s'" % (text.errno, strerror,
+                                                         filename)
             # the exception might have a localized error string encoded with
             # ANSI codepage if OSError and IOError on Windows
             try:
@@ -71,7 +137,7 @@ def to_unicode(text, charset=None):
             return unicode(text)
         except UnicodeError:
             # unicode arguments given to the exception (e.g. parse_date)
-            return ' '.join([to_unicode(arg) for arg in text.args])
+            return ' '.join(to_unicode(arg) for arg in text.args)
     return unicode(text)
 
 
@@ -99,8 +165,8 @@ def path_to_unicode(path):
     return unicode(path)
 
 
-_ws_leading_re = re.compile(ur'\A[\s\u200b]+', re.UNICODE)
-_ws_trailing_re = re.compile(ur'[\s\u200b]+\Z', re.UNICODE)
+_ws_leading_re = re.compile(u'\\A[\\s\u200b]+', re.UNICODE)
+_ws_trailing_re = re.compile(u'[\\s\u200b]+\\Z', re.UNICODE)
 
 def stripws(text, leading=True, trailing=True):
     """Strips unicode white-spaces and ZWSPs from ``text``.
@@ -135,10 +201,10 @@ def strip_line_ws(text, leading=True, trailing=True):
 
 _js_quote = {'\\': '\\\\', '"': '\\"', '\b': '\\b', '\f': '\\f',
              '\n': '\\n', '\r': '\\r', '\t': '\\t', "'": "\\'"}
-for i in range(0x20) + [ord(c) for c in u'&<>\u2028\u2029']:
+for i in list(xrange(0x20)) + [ord(c) for c in u'&<>\u2028\u2029']:
     _js_quote.setdefault(unichr(i), '\\u%04x' % i)
-_js_quote_re = re.compile(ur'[\x00-\x1f\\"\b\f\n\r\t\'&<>\u2028\u2029]')
-_js_string_re = re.compile(ur'[\x00-\x1f\\"\b\f\n\r\t&<>\u2028\u2029]')
+_js_quote_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t\'&<>' + u'\u2028\u2029]')
+_js_string_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t&<>' + u'\u2028\u2029]')
 
 
 def javascript_quote(text):
@@ -270,8 +336,8 @@ def console_print(out, *args, **kwargs):
                    (defaults to `True`)
     """
     cons_charset = stream_encoding(out)
-    out.write(' '.join([to_unicode(a).encode(cons_charset, 'replace')
-                        for a in args]))
+    out.write(' '.join(to_unicode(a).encode(cons_charset, 'replace')
+                       for a in args))
     if kwargs.get('newline', True):
         out.write('\n')
 
@@ -286,6 +352,26 @@ def printerr(*args, **kwargs):
     console_print(sys.stderr, *args, **kwargs)
 
 
+def printfout(message, *args, **kwargs):
+    """Format `message`, do a `console.print` on `sys.stdout` and flush
+    the buffer.
+    """
+    if args:
+        message %= args
+    printout(message, **kwargs)
+    sys.stdout.flush()
+
+
+def printferr(message, *args, **kwargs):
+    """Format `message`, do a `console.print` on `sys.stderr` and flush
+    the buffer.
+    """
+    if args:
+        message %= args
+    printerr(message, **kwargs)
+    sys.stderr.flush()
+
+
 def raw_input(prompt):
     """Input one line from the console and converts it to unicode as
     appropriate.
@@ -493,12 +579,12 @@ class UnicodeTextWrapper(textwrap.TextWrapper):
             pattern = u'[%s]+' % char_ranges
 
         cls.split_re = re.compile(
-            ur'(\s+|' +                                 # any whitespace
+            r'(\s+|' +                                  # any whitespace
             pattern + u'|' +                            # breakable text
-            ur'[^\s\w]*\w+[^0-9\W]-(?=\w+[^0-9\W])|' +  # hyphenated words
-            ur'(?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w))',    # em-dash
+            r'[^\s\w]*\w+[^0-9\W]-(?=\w+[^0-9\W])|' +   # hyphenated words
+            r'(?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w))',     # em-dash
             re.UNICODE)
-        cls.breakable_re = re.compile(ur'\A' + pattern, re.UNICODE)
+        cls.breakable_re = re.compile(r'\A' + pattern, re.UNICODE)
 
     def __init__(self, cols, replace_whitespace=0, break_long_words=0,
                  initial_indent='', subsequent_indent='', ambiwidth=1):
@@ -668,8 +754,8 @@ def pretty_size(size, format='%.1f'):
 
     jump = 1024
     if size < jump:
-        from trac.util.translation import _
-        return _('%(size)s bytes', size=size)
+        from trac.util.translation import ngettext
+        return ngettext("%(num)d byte", "%(num)d bytes", num=size)
 
     units = ['KB', 'MB', 'GB', 'TB']
     i = 0
@@ -746,7 +832,7 @@ def levenshtein_distance(lhs, rhs):
     if not lhs:
         return len(rhs)
 
-    prev = range(len(rhs) + 1)
+    prev = xrange(len(rhs) + 1)
     for lidx, lch in enumerate(lhs):
         curr = [lidx + 1]
         for ridx, rch in enumerate(rhs):
