@@ -1,39 +1,39 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import os.path
 import sys
 
-from genshi.builder import tag
-
 from trac.admin import IAdminCommandProvider, IAdminPanelProvider
+from trac.api import IEnvironmentSetupParticipant
 from trac.config import ListOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.util import as_bool, is_path_below
+from trac.util.html import tag
 from trac.util.text import breakable_path, normalize_whitespace, print_table, \
                            printerr, printout
 from trac.util.translation import _, ngettext, tag_
 from trac.versioncontrol import DbRepositoryProvider, InvalidRepository, \
                                 NoSuchChangeset, RepositoryManager, is_default
-from trac.versioncontrol.cache import CachedRepository
 from trac.web.chrome import Chrome, add_notice, add_warning
 
 
 class VersionControlAdmin(Component):
     """trac-admin command provider for version control administration."""
 
-    implements(IAdminCommandProvider, IPermissionRequestor)
+    implements(IAdminCommandProvider, IEnvironmentSetupParticipant,
+               IPermissionRequestor)
 
     # IAdminCommandProvider methods
 
@@ -66,7 +66,11 @@ class VersionControlAdmin(Component):
                If synchronization gets interrupted, it can be resumed later
                using the `sync` command.
 
-               To synchronize all repositories, specify "*" as the repository.
+               <repos> must be the repository name, not the repository path.
+               Use `list` to see a list of repository names and associated
+               paths. To synchronize all repositories, specify "*" for
+               <repos>. The default repository can be specified
+               using "(default)".
                """,
                self._complete_repos, self._do_resync)
         yield ('repository sync', '<repos> [rev]',
@@ -82,8 +86,8 @@ class VersionControlAdmin(Component):
 
     def get_reponames(self):
         rm = RepositoryManager(self.env)
-        return [reponame or '(default)' for reponame
-                in rm.get_all_repositories()]
+        return [reponame or '(default)'
+                for reponame in rm.get_all_repositories()]
 
     def _complete_repos(self, args):
         if len(args) == 1:
@@ -110,7 +114,7 @@ class VersionControlAdmin(Component):
     def _do_list(self):
         rm = RepositoryManager(self.env)
         values = []
-        for (reponame, info) in sorted(rm.get_all_repositories().iteritems()):
+        for reponame, info in sorted(rm.get_all_repositories().iteritems()):
             alias = ''
             if 'alias' in info:
                 alias = info['alias'] or '(default)'
@@ -130,27 +134,25 @@ class VersionControlAdmin(Component):
                 reponame = ''
             repos = rm.get_repository(reponame)
             if repos is None:
-                raise TracError(_("Repository '%(repo)s' not found",
+                raise TracError(_("Repository \"%(repo)s\" doesn't exist",
                                   repo=reponame or '(default)'))
-            if rev is not None:
-                repos.sync_changeset(rev)
-                printout(_('%(rev)s resynced on %(reponame)s.', rev=rev,
-                           reponame=repos.reponame or '(default)'))
-                return
             repositories = [repos]
 
         for repos in sorted(repositories, key=lambda r: r.reponame):
-            printout(_('Resyncing repository history for %(reponame)s... ',
-                       reponame=repos.reponame or '(default)'))
-            repos.sync(self._sync_feedback, clean=clean)
-            for cnt, in self.env.db_query(
-                    "SELECT count(rev) FROM revision WHERE repos=%s",
-                    (repos.id,)):
-                printout(ngettext('%(num)s revision cached.',
-                                  '%(num)s revisions cached.', num=cnt))
-            if not isinstance(repos, CachedRepository):
-                printout(_("%(reponame)s is not a cached repository.",
-                           reponame=repos.reponame or '(default)'))
+            pretty_name = repos.reponame or '(default)'
+            if rev is not None:
+                repos.sync_changeset(rev)
+                printout(_('%(rev)s resynced on %(reponame)s.', rev=rev,
+                           reponame=pretty_name))
+            else:
+                printout(_('Resyncing repository history for %(reponame)s... ',
+                           reponame=pretty_name))
+                repos.sync(self._sync_feedback, clean=clean)
+                for cnt, in self.env.db_query(
+                        "SELECT count(rev) FROM revision WHERE repos=%s",
+                        (repos.id,)):
+                    printout(ngettext('%(num)s revision cached.',
+                                      '%(num)s revisions cached.', num=cnt))
         printout(_('Done.'))
 
     def _sync_feedback(self, rev):
@@ -163,6 +165,39 @@ class VersionControlAdmin(Component):
     def _do_sync(self, reponame, rev=None):
         self._sync(reponame, rev, clean=False)
 
+    # IEnvironmentSetupParticipant methods
+
+    def environment_created(self):
+        """Index the repositories."""
+        for repos in RepositoryManager(self.env).get_real_repositories():
+            pretty_name = repos.reponame or '(default)'
+            printout(_(" Indexing '%(name)s' repository", name=pretty_name))
+            try:
+                repos.sync(self._sync_feedback)
+            except TracError:
+                printerr(_("""
+ ---------------------------------------------------------------------
+ Warning: couldn't index '%(pretty_name)s' repository.
+
+ This can happen for a variety of reasons: wrong repository type,
+ no appropriate third party library for this repository type,
+ no repository at the specified repository path...
+
+ You can nevertheless start using your Trac environment, but you'll
+ need to check your `%(name)s.type` and `%(name)s.dir` option values
+ in the [repositories] section of your trac.ini file.
+""", pretty_name=pretty_name, name=repos.reponame or ''))
+            else:
+                # Erase to end of line.
+                sys.stdout.write('\033[K')
+                sys.stdout.flush()
+
+    def environment_needs_upgrade(self):
+        pass
+
+    def upgrade_environment(self):
+        pass
+
     # IPermissionRequestor methods
 
     def get_permission_actions(self):
@@ -180,7 +215,8 @@ class RepositoryAdminPanel(Component):
         doc="""Comma-separated list of allowed prefixes for repository
         directories when adding and editing repositories in the repository
         admin panel. If the list is empty, all repository directories are
-        allowed. (''since 0.12.1'')""")
+        allowed.
+        """)
 
     # IAdminPanelProvider methods
 
@@ -225,9 +261,9 @@ class RepositoryAdminPanel(Component):
                         db_provider.modify_repository(reponame, changes)
                         add_notice(req, _('Your changes have been saved.'))
                         name = req.args.get('name')
+                        pretty_name = name or '(default)'
                         resync = tag.code('trac-admin "%s" repository resync '
-                                          '"%s"' % (self.env.path,
-                                                    name or '(default)'))
+                                          '"%s"' % (self.env.path, pretty_name))
                         if 'dir' in changes:
                             msg = tag_('You should now run %(resync)s to '
                                        'synchronize Trac with the repository.',
@@ -238,11 +274,11 @@ class RepositoryAdminPanel(Component):
                                        'synchronize Trac with the repository.',
                                        resync=resync)
                             add_notice(req, msg)
-                        if name and name != path_info and not 'alias' in info:
+                        if name and name != path_info and 'alias' not in info:
                             cset_added = tag.code('trac-admin "%s" changeset '
                                                   'added "%s" $REV'
                                                   % (self.env.path,
-                                                     name or '(default)'))
+                                                     pretty_name))
                             msg = tag_('You will need to update your '
                                        'post-commit hook to call '
                                        '%(cset_added)s with the new '
@@ -252,7 +288,9 @@ class RepositoryAdminPanel(Component):
                     if valid:
                         req.redirect(req.href.admin(category, page))
 
-            Chrome(self.env).add_wiki_toolbars(req)
+            chrome = Chrome(self.env)
+            chrome.add_wiki_toolbars(req)
+            chrome.add_auto_preview(req)
             data = {'view': 'detail', 'reponame': reponame}
 
         else:
@@ -261,6 +299,10 @@ class RepositoryAdminPanel(Component):
                 # Add a repository
                 if db_provider and req.args.get('add_repos'):
                     name = req.args.get('name')
+                    pretty_name = name or '(default)'
+                    if name in all_repos:
+                        raise TracError(_('The repository "%(name)s" already '
+                                          'exists.', name=pretty_name))
                     type_ = req.args.get('type')
                     # Avoid errors when copy/pasting paths
                     dir = normalize_whitespace(req.args.get('dir', ''))
@@ -268,24 +310,18 @@ class RepositoryAdminPanel(Component):
                         add_warning(req, _('Missing arguments to add a '
                                            'repository.'))
                     elif self._check_dir(req, dir):
-                        try:
-                            db_provider.add_repository(name, dir, type_)
-                        except self.env.db_exc.IntegrityError:
-                            name = name or '(default)'
-                            raise TracError(_('The repository "%(name)s" '
-                                              'already exists.', name=name))
-                        name = name or '(default)'
+                        db_provider.add_repository(name, dir, type_)
                         add_notice(req, _('The repository "%(name)s" has been '
-                                          'added.', name=name))
+                                          'added.', name=pretty_name))
                         resync = tag.code('trac-admin "%s" repository resync '
-                                           '"%s"' % (self.env.path, name))
+                                          '"%s"' % (self.env.path, pretty_name))
                         msg = tag_('You should now run %(resync)s to '
                                    'synchronize Trac with the repository.',
                                    resync=resync)
                         add_notice(req, msg)
                         cset_added = tag.code('trac-admin "%s" changeset '
                                               'added "%s" $REV'
-                                              % (self.env.path, name))
+                                              % (self.env.path, pretty_name))
                         doc = tag.a(_("documentation"),
                                     href=req.href.wiki('TracRepositoryAdmin')
                                          + '#Synchronization')
@@ -295,28 +331,27 @@ class RepositoryAdminPanel(Component):
                                    '%(doc)s for more information.',
                                    cset_added=cset_added, doc=doc)
                         add_notice(req, msg)
-                        req.redirect(req.href.admin(category, page))
 
                 # Add a repository alias
                 elif db_provider and req.args.get('add_alias'):
                     name = req.args.get('name')
+                    pretty_name = name or '(default)'
                     alias = req.args.get('alias')
                     if name is not None and alias is not None:
                         try:
                             db_provider.add_alias(name, alias)
                         except self.env.db_exc.IntegrityError:
                             raise TracError(_('The alias "%(name)s" already '
-                                              'exists.',
-                                              name=name or '(default)'))
+                                              'exists.', name=pretty_name))
                         add_notice(req, _('The alias "%(name)s" has been '
-                                          'added.', name=name or '(default)'))
-                        req.redirect(req.href.admin(category, page))
-                    add_warning(req, _('Missing arguments to add an '
-                                       'alias.'))
+                                          'added.', name=pretty_name))
+                    else:
+                        add_warning(req, _('Missing arguments to add an '
+                                           'alias.'))
 
                 # Refresh the list of repositories
                 elif req.args.get('refresh'):
-                    req.redirect(req.href.admin(category, page))
+                    pass
 
                 # Remove repositories
                 elif db_provider and req.args.get('remove'):
@@ -326,8 +361,10 @@ class RepositoryAdminPanel(Component):
                             db_provider.remove_repository(name)
                         add_notice(req, _('The selected repositories have '
                                           'been removed.'))
-                        req.redirect(req.href.admin(category, page))
-                    add_warning(req, _('No repositories were selected.'))
+                    else:
+                        add_warning(req, _('No repositories were selected.'))
+
+                req.redirect(req.href.admin(category, page))
 
             data = {'view': 'list'}
 
@@ -337,13 +374,16 @@ class RepositoryAdminPanel(Component):
             db_repos = dict(db_provider.get_repositories())
 
         # Prepare common rendering data
-        repositories = dict((reponame, self._extend_info(reponame, info.copy(),
-                                                         reponame in db_repos))
-                            for (reponame, info) in all_repos.iteritems())
+        repositories = {reponame: self._extend_info(reponame, info.copy(),
+                                                    reponame in db_repos)
+                        for (reponame, info) in all_repos.iteritems()}
         types = sorted([''] + rm.get_supported_types())
-        data.update({'types': types,
-                     'default_type': rm.default_repository_type,
-                     'repositories': repositories})
+        data.update(
+            {'types': types,
+             'default_type': rm.default_repository_type,
+             'repositories': repositories,
+             'can_add_alias': any('alias' not in info
+                                  for info in repositories.itervalues())})
 
         return 'admin_repositories.html', data
 
