@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2014 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christian Boos <cboos@edgewall.org>
@@ -20,18 +20,31 @@ from itertools import izip
 from tempfile import TemporaryFile
 from zipfile import ZipFile, ZIP_DEFLATED
 
-from genshi.builder import tag
-
 from trac.resource import ResourceNotFound
 from trac.util import content_disposition, create_zipinfo
 from trac.util.datefmt import http_date
+from trac.util.html import tag
 from trac.util.translation import tag_, _
 from trac.versioncontrol.api import EmptyChangeset, NoSuchChangeset, \
                                     NoSuchNode
 from trac.web.api import RequestDone
 
-__all__ = ['get_changes', 'get_path_links', 'get_existing_node',
-           'get_allowed_node', 'make_log_graph', 'render_zip']
+__all__ = ['content_closing', 'get_changes', 'get_path_links',
+           'get_existing_node', 'get_allowed_node', 'make_log_graph',
+           'render_zip']
+
+
+class content_closing(object):
+
+    def __init__(self, content):
+        self.content = content
+
+    def __enter__(self):
+        return self.content
+
+    def __exit__(self, *exc_info):
+        if hasattr(self.content, 'close'):
+            self.content.close()
 
 
 def get_changes(repos, revs, log=None):
@@ -44,7 +57,8 @@ def get_changes(repos, revs, log=None):
         except NoSuchChangeset:
             changeset = EmptyChangeset(repos, rev)
             if log is not None:
-                log.warning("Unable to get changeset [%s]", rev)
+                log.warning("Unable to get changeset [%s] in %s", rev,
+                            repos.reponame or '(default)')
         changes[rev] = changeset
     return changes
 
@@ -120,7 +134,7 @@ def make_log_graph(repos, revs):
             thread.append([1, column, line])
 
     try:
-        next_rev = revs.next()
+        next_rev = next(revs)
         line = 0
         active = []
         active_thread = []
@@ -136,18 +150,18 @@ def make_log_graph(repos, revs):
             column = active.index(rev)
             vertices.append((column, threads.index(active_thread[column])))
 
-            next_rev = revs.next()  # Raises StopIteration when no more revs
-            next = active[:]
+            next_rev = next(revs)  # Raises StopIteration when no more revs
+            next_revs = active[:]
             parents = list(repos.parent_revs(rev))
 
             # Replace current item with parents not already present
             new_parents = [p for p in parents if p not in active]
-            next[column:column + 1] = new_parents
+            next_revs[column:column + 1] = new_parents
 
             # Add edges to parents
             for col, (r, thread) in enumerate(izip(active, active_thread)):
-                if r in next:
-                    add_edge(thread, next.index(r), line + 1)
+                if r in next_revs:
+                    add_edge(thread, next_revs.index(r), line + 1)
                 elif r == rev:
                     if new_parents:
                         parents.remove(new_parents[0])
@@ -155,7 +169,7 @@ def make_log_graph(repos, revs):
                     for parent in parents:
                         if parent != parents[0]:
                             thread.append([0, col, line])
-                        add_edge(thread, next.index(parent), line + 1)
+                        add_edge(thread, next_revs.index(parent), line + 1)
 
             if not new_parents:
                 del active_thread[column]
@@ -165,7 +179,7 @@ def make_log_graph(repos, revs):
                                for i in xrange(len(new_parents) - 1))
                 active_thread[column + 1:column + 1] = threads[base:]
 
-            active = next
+            active = next_revs
             line += 1
     except StopIteration:
         pass
@@ -212,36 +226,33 @@ def render_zip(req, filename, repos, root_node, iter_nodes):
         return end
 
     pos = 0
-    fileobj = TemporaryFile(prefix='trac-', suffix='.zip')
-    try:
-        zipfile = ZipFile(fileobj, 'w', ZIP_DEFLATED)
-        for node in iter_nodes(root_node):
-            if node is root_node:
-                continue
-            path = node.path.strip('/')
-            assert path.startswith(root_path)
-            path = root_name + path[root_len:]
-            kwargs = {'mtime': node.last_modified}
-            data = None
-            if node.isfile:
-                data = node.get_processed_content(eol_hint='CRLF').read()
-                properties = node.get_properties()
-                # Subversion specific
-                if 'svn:special' in properties and data.startswith('link '):
-                    data = data[5:]
-                    kwargs['symlink'] = True
-                if 'svn:executable' in properties:
-                    kwargs['executable'] = True
-            elif node.isdir and path:
-                kwargs['dir'] = True
-                data = ''
-            if data is not None:
-                zipfile.writestr(create_zipinfo(path, **kwargs), data)
-                pos = write_partial(fileobj, pos)
-    finally:
-        try:
-            zipfile.close()
-            write_partial(fileobj, pos)
-        finally:
-            fileobj.close()
+    with TemporaryFile(prefix='trac-', suffix='.zip') as fileobj:
+        with ZipFile(fileobj, 'w', ZIP_DEFLATED) as zipfile:
+            for node in iter_nodes(root_node):
+                if node is root_node:
+                    continue
+                path = node.path.strip('/')
+                assert path.startswith(root_path)
+                path = root_name + path[root_len:]
+                kwargs = {'mtime': node.last_modified}
+                data = None
+                if node.isfile:
+                    with content_closing(
+                            node.get_processed_content(eol_hint='CRLF')) \
+                            as content:
+                        data = content.read()
+                    props = node.get_properties()
+                    # Subversion specific
+                    if 'svn:special' in props and data.startswith('link '):
+                        data = data[5:]
+                        kwargs['symlink'] = True
+                    if 'svn:executable' in props:
+                        kwargs['executable'] = True
+                elif node.isdir and path:
+                    kwargs['dir'] = True
+                    data = ''
+                if data is not None:
+                    zipfile.writestr(create_zipinfo(path, **kwargs), data)
+                    pos = write_partial(fileobj, pos)
+        write_partial(fileobj, pos)
     raise RequestDone
