@@ -1,43 +1,42 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+from abc import ABCMeta
 from BaseHTTPServer import BaseHTTPRequestHandler
 from Cookie import CookieError, BaseCookie, SimpleCookie
 import cgi
 from datetime import datetime
-import errno
-from hashlib import md5
-import new
+import hashlib
+import io
 import mimetypes
 import os
 import re
-import socket
-from StringIO import StringIO
 import sys
+import urllib
 import urlparse
 
-from genshi.builder import Fragment
-from trac.core import Interface, TracError
-from trac.perm import PermissionError
-from trac.util import get_last_traceback, unquote
+from trac.core import Interface, TracBaseError, TracError
+from trac.util import as_bool, as_int, get_last_traceback, lazy, \
+                      normalize_filename
 from trac.util.datefmt import http_date, localtz
+from trac.util.html import Fragment, tag
 from trac.util.text import empty, exception_to_unicode, to_unicode
-from trac.util.translation import _
+from trac.util.translation import _, N_, tag_
 from trac.web.href import Href
-from trac.web.wsgi import _FileWrapper
+from trac.web.wsgi import _FileWrapper, is_client_disconnect_exception
 
 
 class IAuthenticator(Interface):
@@ -50,7 +49,19 @@ class IAuthenticator(Interface):
 
 
 class IRequestHandler(Interface):
-    """Decide which `trac.core.Component` handles which `Request`, and how."""
+    """Decide which `trac.core.Component` handles which `Request`, and how.
+
+    The boolean property `is_valid_default_handler` determines whether the
+    `IRequestFilter` can be used as a `default_handler` and defaults to
+    `True`. To be suitable as a `default_handler`, an `IRequestFilter` must
+    return an HTML document and `data` dictionary for rendering the document,
+    and must not require that `match_request` be called prior to
+    `process_request`.
+
+    The boolean property `jquery_noconflict` determines whether jQuery's
+    `noConflict` mode will be activated by the handler, and defaults to
+    `False`.
+    """
 
     def match_request(req):
         """Return whether the handler wants to process the given request."""
@@ -58,18 +69,65 @@ class IRequestHandler(Interface):
     def process_request(req):
         """Process the request.
 
-        Return a `(template_name, data, content_type)` tuple,
-        where `data` is a dictionary of substitutions for the Genshi template.
+        Return a `(template_name, data)` pair, where `data` is a
+        dictionary of substitutions for the Jinja2 template (the
+        template context, in Jinja2 terms).
 
-        "text/html" is assumed if `content_type` is `None`.
+        Optionally, the return value can also be a `(template_name,
+        data, metadata)` triple, where `metadata` is a `dict` with
+        hints for the template engine or the web front-end.
 
-        Note that if template processing should not occur, this method can
-        simply send the response itself and not return anything.
+        Keys supported are:
+
+          - `'content_type'`: the mimetype used for content delivery;
+            "text/html" is assumed if the key is not present or the
+            `metadata` was not specified
+
+          - `'text'`: a boolean value indicating whether the Jinja2
+            auto-escaping feature should be deactivated
+            (``text=True``) or not (``text=False``); defaults to
+            ``False``, suitable for generating HTML or XML content
+
+          - `'fragment'`: a boolean value indicating whether the
+            generated content will be used as part of another page
+            (``fragment=True``) or as a stand-alone page
+            (``fragment=False``), the default
+
+          - `'domain'`: a string value indicating the translation
+            domain to which the translated strings in the template
+            belong to
+
+        Note that if template processing should not occur, this method
+        can simply send the response itself (see `Request` methods)
+        and not return anything, as the `Request` methods raise a
+        `RequestDone` exception.
 
         :Since 1.0: Clearsilver templates are no longer supported.
+
+        :Since 1.1.2: the rendering `method` (xml, xhtml or text) may be
+           returned as a fourth parameter in the tuple, but if not specified
+           it will be inferred from the `content_type` when rendering the
+           template.
+
+        :Since 1.3.2: returns a pair, or a tuple in which the third
+           element is a `dict` instead of a string like in the old
+           API.  Note that the old API (`(template, data,
+           content_type)` where `content_type` is a string or `None`)
+           is still supported. When used, this means that `template`
+           is a legacy Genshi template. This support for the old API
+           will be removed in Trac 1.5.1, in which `metadata` will
+           always be a `dict` or `None` when specified.
+
         """
 
 
+def is_valid_default_handler(handler):
+    """Returns `True` if the `handler` is a valid default handler, as
+    described in the `IRequestHandler` interface documentation.
+    """
+    return handler and getattr(handler, 'is_valid_default_handler', True)
+
+
 class IRequestFilter(Interface):
     """Enable components to interfere with the processing done by the
     main handler, either before and/or after it enters in action.
@@ -82,33 +140,62 @@ class IRequestFilter(Interface):
         Always returns the request handler, even if unchanged.
         """
 
-    def post_process_request(req, template, data, content_type):
-        """Do any post-processing the request might need; typically adding
-        values to the template `data` dictionary, or changing the Genshi
-        template or mime type.
+    def post_process_request(req, template, data, metadata=None, method=None):
+        """Do any post-processing the request might need
 
-        `data` may be updated in place.
+        This typically means adding values to the template `data`
+        dictionary, or changing the Jinja2 template.
 
-        Always returns a tuple of (template, data, content_type), even if
-        unchanged.
+        `data` and `metdata` may be updated in place.
+
+        Always returns a tuple of ``(template, data)`` or ``(template,
+        data, metadata)``, even if unchanged.
+
+        Be aware that returning a ``(template, data, None)`` triple
+        will be interpreted as using the legacy API and will indicate
+        that the template is a legacy Genshi template. The same will
+        happen if the third value of the tuple is a string.
+
+        The `method` last parameter is deprecated and is now supposed
+        to be passed as `'method'` in the `metadata` dict. Even
+        better, use directly the `'text'` indication (`True` or
+        `False`).
 
         Note that `template`, `data`, `content_type` will be `None` if:
          - called when processing an error page
          - the default request handler did not return any result
 
-        :Since 0.11: there's a `data` argument for supporting Genshi templates;
-           this introduced a difference in arity which made it possible to
-           distinguish between the IRequestFilter components still targeted
-           at ClearSilver templates and the newer ones targeted at Genshi
-           templates.
-
         :Since 1.0: Clearsilver templates are no longer supported.
+
+        :Since 1.1.2: the rendering `method` will be passed if it is returned
+           by the request handler, otherwise `method` will be `None`. For
+           backward compatibility, the parameter is optional in the
+           implementation's signature.
+
+        :Since 1.3.2: Genshi templates are still supported, and if
+           `process_request` used the old API (`(template, data,
+           content_type)`), the `metadata` parameter passed to
+           `post_process_request` will actually be the `content_type`
+           value (`String` or `None`).  This support for the old API
+           will be removed in Trac 1.5.1, in which `metadata` will
+           always be a `dict` or `None`.
+
         """
 
 
 class ITemplateStreamFilter(Interface):
     """Transform the generated content by filtering the Genshi event stream
     generated by the template, prior to its serialization.
+
+    :deprecated: the Genshi template filtering concept doesn't apply
+                 anymore to Jinja2 templates, please consider
+                 converting your plugins to perform browser-side
+                 modifications of the rendered page using JavaScript.
+                 This interface will be removed in Trac 1.5.1.
+
+    See :teo:`TracDev/PortingFromGenshiToJinja#ReplacingITemplateStreamFilter`
+    for details.
+
     """
 
     def filter_stream(req, method, filename, stream, data):
@@ -126,32 +213,49 @@ class ITemplateStreamFilter(Interface):
         """
 
 
-HTTP_STATUS = dict([(code, reason.title()) for code, (reason, description)
-                    in BaseHTTPRequestHandler.responses.items()])
+class TracNotImplementedError(TracError, NotImplementedError):
+    """Raised when a `NotImplementedError` is trapped.
+
+    This exception is for internal use and should not be raised by
+    plugins. Plugins should raise `NotImplementedError`.
+
+    :since: 1.0.11
+    """
+
+    title = N_("Not Implemented Error")
 
 
-class HTTPException(Exception):
+HTTP_STATUS = {code: reason.title()
+               for code, (reason, description)
+               in BaseHTTPRequestHandler.responses.items()}
+
+
+class HTTPException(TracBaseError):
+
+    __metaclass__ = ABCMeta
+
+    code = None
 
     def __init__(self, detail, *args):
-        if isinstance(detail, (TracError, PermissionError)):
+        """Factory for HTTPException classes."""
+        if isinstance(detail, TracBaseError):
             self.detail = detail.message
             self.reason = detail.title
         else:
             self.detail = detail
         if args:
             self.detail = self.detail % args
-        super(HTTPException, self).__init__('%s %s (%s)' % (self.code,
-                                                            self.reason,
-                                                            self.detail))
+        arg = u'%s %s (%s)' % (self.code, self.reason, to_unicode(self.detail))
+        super(HTTPException, self).__init__(arg)
 
     @property
     def message(self):
         # The message is based on the e.detail, which can be an Exception
         # object, but not a TracError one: when creating HTTPException,
         # a TracError.message is directly assigned to e.detail
-        if isinstance(self.detail, Exception): # not a TracError or PermissionError
+        if isinstance(self.detail, Exception): # not a TracBaseError
             message = exception_to_unicode(self.detail)
-        elif isinstance(self.detail, Fragment): # TracError or PermissionError markup
+        elif isinstance(self.detail, Fragment): # TracBaseError markup
             message = self.detail
         else:
             message = to_unicode(self.detail)
@@ -177,7 +281,7 @@ class HTTPException(Exception):
     def subclass(cls, name, code):
         """Create a new Exception class representing a HTTP status code."""
         reason = HTTP_STATUS.get(code, 'Unknown')
-        new_class = new.classobj(name, (HTTPException,), {
+        new_class = type(name, (HTTPException,), {
             '__doc__': 'Exception for HTTP %d %s' % (code, reason)
         })
         new_class.code = code
@@ -187,9 +291,6 @@ class HTTPException(Exception):
 _HTTPException_subclass_names = []
 for code in [code for code in HTTP_STATUS if code >= 400]:
     exc_name = HTTP_STATUS[code].replace(' ', '').replace('-', '')
-    # 2.5 compatibility hack:
-    if exc_name == 'InternalServerError':
-        exc_name = 'InternalError'
     if exc_name.lower().startswith('http'):
         exc_name = exc_name[4:]
     exc_name = 'HTTP' + exc_name
@@ -215,9 +316,98 @@ class _RequestArgs(dict):
     """Dictionary subclass that provides convenient access to request
     parameters that may contain multiple values."""
 
+    def get(self, name, default=None):
+        """Return the first value for the specified parameter, or `default`
+        if the parameter was not provided.
+
+        :since 1.3.2: aliases `getfirst`. Use `getlist` if the value is
+                      expected to be a list.
+        """
+        return self.getfirst(name, default)
+
+    def as_int(self, name, default=None, min=None, max=None):
+        """Return the value as an integer. Return `default` if
+        if an exception is raised while converting the value to an
+        integer.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified or an exception occurs converting
+                          the value to an integer.
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        return as_int(self.get(name), default, min, max)
+
+    def as_bool(self, name, default=None):
+        """Return the value as a boolean. Return `default` if
+        if an exception is raised while converting the value to a
+        boolean.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified or an exception occurs converting
+                          the value to a boolean.
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        return as_bool(self.get(name), default)
+
+    def getbool(self, name, default=None):
+        """Return the value as a boolean. Raise an `HTTPBadRequest`
+        exception if an exception occurs while converting the value to
+        a boolean.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified.
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        value = self[name]
+        if isinstance(value, list):
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        value = as_bool(value, None)
+        if value is None:
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        return value
+
+    def getint(self, name, default=None, min=None, max=None):
+        """Return the value as an integer. Raise an `HTTPBadRequest`
+        exception if an exception occurs while converting the value
+        to an integer.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        value = as_int(self[name], None, min, max)
+        if value is None:
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        return value
+
     def getfirst(self, name, default=None):
-        """Return the first value for the specified parameter, or `default` if
-        the parameter was not provided.
+        """Return the first value for the specified parameter, or `default`
+        if the parameter was not provided.
+
+        :since 1.3.2: `get` aliases `getfirst` and should be used instead.
         """
         if name not in self:
             return default
@@ -237,22 +427,73 @@ class _RequestArgs(dict):
             val = [val]
         return val
 
+    def getfile(self, name):
+        """Return a tuple of the filename, file object and file size.
+
+        :param name: the name of the request parameter
+
+        :since: 1.3.3
+        """
+        upload = self.getfirst(name)
+        return self._getfile(upload)
+
+    def getfilelist(self, name):
+        """Return a list of tuples containing the filename, file object
+        and file size.
+
+        :param name: the name of the request parameter
+
+        :since: 1.3.3
+        """
+        return [self._getfile(upload) for upload in self.getlist(name)]
+
+    def require(self, name):
+        """Raise an `HTTPBadRequest` exception if the parameter is
+        not in the request.
+
+        :param name: the name of the request parameter
+
+        :since: 1.2
+        """
+        if name not in self:
+            raise HTTPBadRequest(
+                tag_("Missing request argument. The %(name)s argument "
+                     "must be included in the request.", name=tag.em(name)))
+
+    def _getfile(self, upload):
+        filename = fileobj = size = None
+        if hasattr(upload, 'filename'):
+            filename = normalize_filename(upload.filename)
+        if hasattr(upload, 'file'):
+            fileobj = upload.file
+            if hasattr(fileobj, 'fileno'):
+                size = os.fstat(fileobj.fileno())[6]
+            else:
+                fileobj.seek(0, 2)  # seek to end of file
+                size = fileobj.tell()
+                fileobj.seek(0)
+
+        return filename, fileobj, size
+
 
 def parse_arg_list(query_string):
-    """Parse a query string into a list of `(name, value)` tuples."""
+    """Parse a query string into a list of `(name, value)` tuples.
+
+    :Since 1.1.2: a leading `?` is stripped from `query_string`."""
     args = []
     if not query_string:
         return args
+    query_string = query_string.lstrip('?')
     for arg in query_string.split('&'):
         nv = arg.split('=', 1)
         if len(nv) == 2:
             (name, value) = nv
         else:
             (name, value) = (nv[0], empty)
-        name = unquote(name.replace('+', ' '))
+        name = urllib.unquote(name.replace('+', ' '))
         if isinstance(name, str):
             name = unicode(name, 'utf-8')
-        value = unquote(value.replace('+', ' '))
+        value = urllib.unquote(value.replace('+', ' '))
         if isinstance(value, str):
             value = unicode(value, 'utf-8')
         args.append((name, value))
@@ -273,7 +514,7 @@ def arg_list_to_args(arg_list):
     return args
 
 
-class RequestDone(Exception):
+class RequestDone(TracBaseError):
     """Marker exception that indicates whether request processing has completed
     and a response was sent.
     """
@@ -314,6 +555,12 @@ class Request(object):
     This class provides a convenience API over WSGI.
     """
 
+    _disallowed_control_codes_re = re.compile(r'[\x00-\x08\x0a-\x1f\x7f]')
+    _reserved_headers = {'content-type', 'content-length', 'location',
+                         'etag', 'pragma', 'cache-control', 'expires'}
+    # RFC7230 3.2 Header Fields
+    _valid_header_re = re.compile(r"[-0-9A-Za-z!#$%&'*+.^_`|~]+\Z")
+
     def __init__(self, environ, start_response):
         """Create the request wrapper.
 
@@ -327,9 +574,9 @@ class Request(object):
         self._write = None
         self._status = '200 OK'
         self._response = None
+        self._content_type = None
 
         self._outheaders = []
-        self._outcharset = None
         self.outcookie = Cookie()
 
         self.callbacks = {
@@ -337,7 +584,8 @@ class Request(object):
             'args': lambda req: arg_list_to_args(req.arg_list),
             'languages': Request._parse_languages,
             'incookie': Request._parse_cookies,
-            '_inheaders': Request._parse_headers
+            '_inheaders': Request._parse_headers,
+            'locale': lambda req: None,  # prevent AttributeError
         }
         self.redirect_listeners = []
 
@@ -357,12 +605,30 @@ class Request(object):
         raise AttributeError(name)
 
     def __repr__(self):
-        path_info = self.environ.get('PATH_INFO', '')
-        return '<%s "%s %r">' % (self.__class__.__name__, self.method,
-                                 path_info)
+        uri = self.environ.get('PATH_INFO', '')
+        qs = self.query_string
+        if qs:
+            uri += '?' + qs
+        return '<%s "%s %r">' % (self.__class__.__name__, self.method, uri)
 
     # Public API
 
+    @lazy
+    def is_authenticated(self):
+        """Returns `True` if `authname` is not `anonymous`.
+
+        :since: 1.3.2
+        """
+        return self.authname and self.authname != 'anonymous'
+
+    @lazy
+    def is_xhr(self):
+        """Returns `True` if the request is an `XMLHttpRequest`.
+
+        :since: 1.1.6
+        """
+        return self.get_header('X-Requested-With') == 'XMLHttpRequest'
+
     @property
     def method(self):
         """The HTTP method of the request"""
@@ -398,6 +664,10 @@ class Request(object):
         if user is not None:
             return to_unicode(user)
 
+    @property
+    def response_started(self):
+        return self._write is not None
+
     @property
     def scheme(self):
         """The scheme of the request URL"""
@@ -445,21 +715,21 @@ class Request(object):
         `value` must either be an `unicode` string or can be converted to one
         (e.g. numbers, ...)
         """
-        lower_name = name.lower()
-        if lower_name == 'content-type':
-            ctpos = value.find('charset=')
-            if ctpos >= 0:
-                self._outcharset = value[ctpos + 8:].strip()
-        elif lower_name == 'content-length':
-            self._content_length = int(value)
+        if name.lower() == 'content-type':
+            self._content_type = value.split(';', 1)[0]
         self._outheaders.append((name, unicode(value).encode('utf-8')))
 
-    def end_headers(self):
+    def end_headers(self, exc_info=None):
         """Must be called after all headers have been sent and before the
         actual content is written.
         """
+        if self.method == 'POST' and self._content_type == 'text/html':
+            # Disable XSS protection (#12926)
+            self.send_header('X-XSS-Protection', 0)
+        self._send_configurable_headers()
         self._send_cookie_headers()
-        self._write = self._start_response(self._status, self._outheaders)
+        self._write = self._start_response(self._status, self._outheaders,
+                                           exc_info)
 
     def check_modified(self, datetime, extra=''):
         """Check the request "If-None-Match" header against an entity tag.
@@ -477,7 +747,7 @@ class Request(object):
         so that consecutive requests can be cached.
         """
         if isinstance(extra, list):
-            m = md5()
+            m = hashlib.sha1()
             for elt in extra:
                 m.update(repr(elt))
             extra = m.hexdigest()
@@ -491,6 +761,8 @@ class Request(object):
             self.end_headers()
             raise RequestDone
 
+    _trident_re = re.compile(r' Trident/([0-9]+)')
+
     def redirect(self, url, permanent=False):
         """Send a redirect to the client, forwarding to the specified URL.
 
@@ -515,9 +787,10 @@ class Request(object):
 
         # Workaround #10382, IE6-IE9 bug when post and redirect with hash
         if status == 303 and '#' in url:
-            match = re.search(' MSIE ([0-9]+)',
-                              self.environ.get('HTTP_USER_AGENT', ''))
-            if match and int(match.group(1)) < 10:
+            user_agent = self.environ.get('HTTP_USER_AGENT', '')
+            match_trident = self._trident_re.search(user_agent)
+            if ' MSIE ' in user_agent and \
+                    (not match_trident or int(match_trident.group(1)) < 6):
                 url = url.replace('#', '#__msie303:')
 
         self.send_header('Location', url)
@@ -530,57 +803,52 @@ class Request(object):
         raise RequestDone
 
     def send(self, content, content_type='text/html', status=200):
-        self.send_response(status)
-        self.send_header('Cache-Control', 'must-revalidate')
-        self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
-        self.send_header('Content-Type', content_type + ';charset=utf-8')
-        self.send_header('Content-Length', len(content))
-        self.end_headers()
-
-        if self.method != 'HEAD':
-            self.write(content)
-        raise RequestDone
+        self._send(content, content_type, status)
 
     def send_error(self, exc_info, template='error.html',
                    content_type='text/html', status=500, env=None, data={}):
+        """Render error page.
+
+        The rendered content can also be passed to the method, when the
+        method is invoked as:
+        `send_error(exc_info, content, content_type='text/html', status=500)`
+
+        :since 1.3.3: the method that renders an error page is deprecated
+            in favor of the method that sends rendered content. The method
+            that renders an error page will be removed in 1.5.1.
+        """
         try:
-            if template.endswith('.html'):
+            if isinstance(template, basestring) and template.endswith('.html'):
                 if env:
-                    from trac.web.chrome import Chrome
+                    from trac.web.chrome import Chrome, add_stylesheet
+                    add_stylesheet(self, 'common/css/code.css')
+                    metadata = {'content_type': 'text/html'}
                     try:
-                        data = Chrome(env).render_template(self, template,
-                                                           data, 'text/html')
+                        content = Chrome(env).render_template(self, template,
+                                                              data, metadata)
                     except Exception:
                         # second chance rendering, in "safe" mode
                         data['trac_error_rendering'] = True
-                        data = Chrome(env).render_template(self, template,
-                                                           data, 'text/html')
+                        content = Chrome(env).render_template(self, template,
+                                                              data, metadata)
                 else:
                     content_type = 'text/plain'
-                    data = '%s\n\n%s: %s' % (data.get('title'),
-                                             data.get('type'),
-                                             data.get('message'))
+                    content = '%s\n\n%s: %s' % (data.get('title'),
+                                                data.get('type'),
+                                                data.get('message'))
+            else:
+                # send_error(self, exc_info, content, content_type='text/html',
+                #            status=500)
+                content = template
         except Exception: # failed to render
-            data = get_last_traceback()
+            content = get_last_traceback()
             content_type = 'text/plain'
 
-        if isinstance(data, unicode):
-            data = data.encode('utf-8')
+        if isinstance(content, unicode):
+            content = content.encode('utf-8')
 
-        self.send_response(status)
         self._outheaders = []
-        self.send_header('Cache-Control', 'must-revalidate')
-        self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
-        self.send_header('Content-Type', content_type + ';charset=utf-8')
-        self.send_header('Content-Length', len(data))
-        self._send_cookie_headers()
-
-        self._write = self._start_response(self._status, self._outheaders,
-                                           exc_info)
-
-        if self.method != 'HEAD':
-            self.write(data)
-        raise RequestDone
+        self._send(content, content_type, status, exc_info)
 
     def send_no_content(self):
         self.send_response(204)
@@ -620,7 +888,11 @@ class Request(object):
         self.send_header('Last-Modified', last_modified)
         use_xsendfile = getattr(self, 'use_xsendfile', False)
         if use_xsendfile:
-            self.send_header('X-Sendfile', os.path.abspath(path))
+            xsendfile_header = getattr(self, 'xsendfile_header', None)
+            if xsendfile_header:
+                self.send_header(xsendfile_header, os.path.abspath(path))
+            else:
+                use_xsendfile = False
         self.end_headers()
 
         if not use_xsendfile and self.method != 'HEAD':
@@ -641,12 +913,15 @@ class Request(object):
         data = fileobj.read(size)
         return data
 
+    CHUNK_SIZE = 4096
+
     def write(self, data):
         """Write the given data to the response body.
 
-        *data* **must** be a `str` string, encoded with the charset
-        which has been specified in the ``'Content-Type'`` header
-        or UTF-8 otherwise.
+        *data* **must** be a `str` string or an iterable instance
+        which iterates `str` strings, encoded with the charset which
+        has been specified in the ``'Content-Type'`` header or UTF-8
+        otherwise.
 
         Note that when the ``'Content-Length'`` header is specified,
         its value either corresponds to the length of *data*, or, if
@@ -655,23 +930,58 @@ class Request(object):
         """
         if not self._write:
             self.end_headers()
-        if isinstance(data, unicode):
-            raise ValueError("Can't send unicode content")
         try:
-            self._write(data)
-        except (IOError, socket.error), e:
-            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
-                raise RequestDone
-            # Note that mod_wsgi raises an IOError with only a message
-            # if the client disconnects
-            if 'mod_wsgi.version' in self.environ and \
-               e.args[0] in ('failed to write data',
-                             'client connection closed'):
+            chunk_size = self.CHUNK_SIZE
+            bufsize = 0
+            buf = []
+            buf_append = buf.append
+            if isinstance(data, basestring):
+                data = [data]
+            for chunk in data:
+                if isinstance(chunk, unicode):
+                    raise ValueError("Can't send unicode content")
+                if not chunk:
+                    continue
+                bufsize += len(chunk)
+                buf_append(chunk)
+                if bufsize >= chunk_size:
+                    self._write(''.join(buf))
+                    bufsize = 0
+                    buf[:] = ()
+            if bufsize > 0:
+                self._write(''.join(buf))
+        except IOError as e:
+            if is_client_disconnect_exception(e):
                 raise RequestDone
             raise
 
+    @classmethod
+    def is_valid_header(cls, name, value=None):
+        """Check whether the field name, and optionally the value, make
+        a valid HTTP header.
+        """
+        valid_name = name and name.lower() not in cls._reserved_headers and \
+                     bool(cls._valid_header_re.match(name))
+        valid_value = not cls._disallowed_control_codes_re.search(value) \
+                      if value else True
+        return valid_name & valid_value
+
     # Internal methods
 
+    def _send(self, content, content_type='text/html', status=200,
+              exc_info=None):
+        self.send_response(status)
+        self.send_header('Cache-Control', 'must-revalidate')
+        self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
+        self.send_header('Content-Type', content_type + ';charset=utf-8')
+        if isinstance(content, basestring):
+            self.send_header('Content-Length', len(content))
+        self.end_headers(exc_info)
+
+        if self.method != 'HEAD':
+            self.write(content)
+        raise RequestDone
+
     def _parse_arg_list(self):
         """Parse the supplied request parameters into a list of
         `(name, value)` tuples.
@@ -685,22 +995,46 @@ class Request(object):
             ctype, options = cgi.parse_header(ctype)
         if ctype not in ('application/x-www-form-urlencoded',
                          'multipart/form-data'):
-            fp = StringIO('')
+            fp = io.BytesIO()
 
         # Python 2.6 introduced a backwards incompatible change for
         # FieldStorage where QUERY_STRING is no longer ignored for POST
         # requests. We'll keep the pre 2.6 behaviour for now...
         if self.method == 'POST':
             qs_on_post = self.environ.pop('QUERY_STRING', '')
-        fs = _FieldStorage(fp, environ=self.environ, keep_blank_values=True)
+        try:
+            fs = _FieldStorage(fp, environ=self.environ,
+                               keep_blank_values=True)
+        except IOError as e:
+            if is_client_disconnect_exception(e):
+                raise HTTPBadRequest(
+                    _("Exception caught while reading request: %(msg)s",
+                      msg=exception_to_unicode(e)))
+            raise
         if self.method == 'POST':
             self.environ['QUERY_STRING'] = qs_on_post
 
+        def raise_if_null_bytes(value):
+            if value and '\x00' in value:
+                raise HTTPBadRequest(_("Invalid request arguments."))
+
         args = []
         for value in fs.list or ():
             name = value.name
-            if not value.filename:
-                value = unicode(value.value, 'utf-8')
+            raise_if_null_bytes(name)
+            try:
+                if name is not None:
+                    name = unicode(name, 'utf-8')
+                if value.filename:
+                    raise_if_null_bytes(value.filename)
+                else:
+                    value = value.value
+                    raise_if_null_bytes(value)
+                    value = unicode(value, 'utf-8')
+            except UnicodeDecodeError as e:
+                raise HTTPBadRequest(
+                    _("Invalid encoding in form data: %(msg)s",
+                      msg=exception_to_unicode(e)))
             args.append((name, value))
         return args
 
@@ -754,8 +1088,14 @@ class Request(object):
         return urlparse.urlunparse((self.scheme, host, self.base_path, None,
                                     None, None))
 
+    def _send_configurable_headers(self):
+        sent_headers = [name.lower() for name, val in self._outheaders]
+        for name, val in getattr(self, 'configurable_headers', []):
+            if name.lower() not in sent_headers:
+                self.send_header(name, val)
+
     def _send_cookie_headers(self):
-        for name in self.outcookie.keys():
+        for name in list(self.outcookie):
             path = self.outcookie[name].get('path')
             if path:
                 path = path.replace(' ', '%20') \
@@ -767,4 +1107,5 @@ class Request(object):
         for cookie in cookies.splitlines():
             self._outheaders.append(('Set-Cookie', cookie.strip()))
 
+
 __no_apidoc__ = _HTTPException_subclass_names
