@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
@@ -25,8 +25,6 @@ import sys
 import urllib2
 import urlparse
 
-from genshi.builder import tag
-
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.web.api import IAuthenticator, IRequestHandler
@@ -35,6 +33,7 @@ from trac.util import hex_entropy, md5crypt
 from trac.util.compat import crypt
 from trac.util.concurrency import threading
 from trac.util.datefmt import time_now
+from trac.util.html import tag
 from trac.util.translation import _, tag_
 
 
@@ -79,12 +78,14 @@ class LoginModule(Component):
         authentication information, and therefore, after how much
         inactivity a user will have to log in again. The value
         of 0 makes the cookie expire at the end of the browsing
-        session. (''since 0.12'')""")
+        session.
+        """)
 
     auth_cookie_path = Option('trac', 'auth_cookie_path', '',
         """Path for the authentication cookie. Set this to the common
         base path of several Trac instances if you want them to share
-        the cookie.  (''since 0.12'')""")
+        the cookie.
+        """)
 
     # IAuthenticator methods
 
@@ -110,15 +111,20 @@ class LoginModule(Component):
         return 'login'
 
     def get_navigation_items(self, req):
-        if req.authname and req.authname != 'anonymous':
+        if req.is_authenticated:
             yield ('metanav', 'login',
                    tag_("logged in as %(user)s",
                         user=Chrome(self.env).authorinfo(req, req.authname)))
             yield ('metanav', 'logout',
-                   tag.form(tag.div(tag.button(_("Logout"),
-                                               name='logout', type='submit')),
-                            action=req.href.logout(), method='post',
-                            id='logout', class_='trac-logout'))
+                   tag.form(
+                       tag.div(
+                           tag.button(_("Logout"), name='logout',
+                                      type='submit'),
+                           tag.input(type='hidden', name='__FORM_TOKEN',
+                                     value=req.form_token)
+                       ),
+                       action=req.href.logout(), method='post',
+                       id='logout', class_='trac-logout'))
         else:
             yield ('metanav', 'login',
                    tag.a(_("Login"), href=req.href.login()))
@@ -204,10 +210,7 @@ class LoginModule(Component):
         Simply deletes the corresponding record from the auth_cookie
         table.
         """
-        if req.method != 'POST':
-            return
-        if req.authname == 'anonymous':
-            # Not logged in
+        if req.method != 'POST' or not req.is_authenticated:
             return
 
         if 'trac_auth' in req.incookie:
@@ -321,23 +324,23 @@ class BasicAuthentication(PasswordFileAuthentication):
     def load(self, filename):
         # FIXME use a logger
         self.hash = {}
-        fd = open(filename, 'r')
-        for line in fd:
-            line = line.split('#')[0].strip()
-            if not line:
-                continue
-            try:
-                u, h = line.split(':')[:2]
-            except ValueError:
-                print("Warning: invalid password line in %s: %s"
-                      % (filename, line), file=sys.stderr)
-                continue
-            if '$' in h or h.startswith('{SHA}') or self.crypt:
-                self.hash[u] = h
-            else:
-                print('Warning: cannot parse password for user "%s" '
-                      'without the "crypt" module. Install the passlib '
-                      'package from PyPI' % u, file=sys.stderr)
+        with open(filename) as fd:
+            for line in fd:
+                line = line.split('#')[0].strip()
+                if not line:
+                    continue
+                try:
+                    u, h = line.split(':')[:2]
+                except ValueError:
+                    print("Warning: invalid password line in %s: %s"
+                          % (filename, line), file=sys.stderr)
+                    continue
+                if '$' in h or h.startswith('{SHA}') or self.crypt:
+                    self.hash[u] = h
+                else:
+                    print('Warning: cannot parse password for user "%s" '
+                          'without the "crypt" module. Install the passlib '
+                          'package from PyPI' % u, file=sys.stderr)
 
         if self.hash == {}:
             print("Warning: found no users in file:", filename,
@@ -392,19 +395,19 @@ class DigestAuthentication(PasswordFileAuthentication):
         """
         # FIXME use a logger
         self.hash = {}
-        fd = open(filename, 'r')
-        for line in fd:
-            line = line.split('#')[0].strip()
-            if not line:
-                continue
-            try:
-                u, r, a1 = line.split(':')[:3]
-            except ValueError:
-                print("Warning: invalid digest line in %s: %s"
-                      % (filename, line), file=sys.stderr)
-                continue
-            if r == self.realm:
-                self.hash[u] = a1
+        with open(filename) as fd:
+            for line in fd:
+                line = line.split('#')[0].strip()
+                if not line:
+                    continue
+                try:
+                    u, r, a1 = line.split(':')[:3]
+                except ValueError:
+                    print("Warning: invalid digest line in %s: %s"
+                          % (filename, line), file=sys.stderr)
+                    continue
+                if r == self.realm:
+                    self.hash[u] = a1
         if self.hash == {}:
             print("Warning: found no users in realm:", self.realm,
                   file=sys.stderr)
