@@ -18,6 +18,8 @@
 
 import re
 
+from jinja2.runtime import Context as Jinja2Context
+
 from trac.util.text import unicode_quote, unicode_urlencode
 
 
@@ -160,6 +162,9 @@ class Href(object):
             elif value is not None:
                 params.append((name, value))
 
+        # Skip Jinja2 context (#13244)
+        if args and isinstance(args[0], Jinja2Context):
+            args = args[1:]
         if args:
             lastp = args[-1]
             if isinstance(lastp, dict):
