@@ -1,21 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 import os.path
 import pkg_resources
 import sys
 
 from trac.admin import *
+from trac.api import IEnvironmentSetupParticipant
 from trac.core import *
 from trac.wiki import model
 from trac.wiki.api import WikiSystem, validate_page_name
@@ -30,7 +31,7 @@ from trac.util.translation import _
 class WikiAdmin(Component):
     """trac-admin command provider for wiki administration."""
 
-    implements(IAdminCommandProvider)
+    implements(IAdminCommandProvider, IEnvironmentSetupParticipant)
 
     # IAdminCommandProvider methods
 
@@ -66,7 +67,7 @@ class WikiAdmin(Component):
                directory are imported.""",
                self._complete_load_replace, self._do_load)
         yield ('wiki replace', '<path> [...]',
-               """Replace the content of wiki pages from files (DANGEROUS!)
+               """Replace content of wiki pages from files (DANGEROUS!)
 
                This command replaces the content of the last version of one
                or more wiki pages with new content. The previous content is
@@ -140,10 +141,10 @@ class WikiAdmin(Component):
                       """, (data, title, title))
             else:
                 db("""INSERT INTO wiki (version, readonly, name, time, author,
-                                        ipnr, text)
+                                        text)
                       SELECT 1 + COALESCE(max(version), 0),
                              COALESCE(max(readonly), 0),
-                             %s, %s, 'trac', '127.0.0.1', %s FROM wiki
+                             %s, %s, 'trac', %s FROM wiki
                       WHERE name=%s AND version=(SELECT max(version)
                                                  FROM wiki WHERE name=%s)
                       """, (title, to_utimestamp(datetime_now(utc)), data,
@@ -161,9 +162,8 @@ class WikiAdmin(Component):
                 page = unicode_unquote(page.encode('utf-8'))
                 if os.path.isfile(filename):
                     if self.import_page(filename, page, create_only, replace):
-                        printout(_("  %(page)s imported from %(filename)s",
-                                   filename=path_to_unicode(filename),
-                                   page=page))
+                        self.log.info("%s imported from %s",
+                                      page, path_to_unicode(filename))
 
     def _complete_page(self, args):
         if len(args) == 1:
@@ -266,4 +266,24 @@ class WikiAdmin(Component):
     def _do_upgrade(self):
         self.load_pages(pkg_resources.resource_filename('trac.wiki',
                                                         'default-pages'),
-                        ignore=['WikiStart'], create_only=['InterMapTxt'])
+                        ignore=['WikiStart', 'SandBox'],
+                        create_only=['InterMapTxt'])
+
+    # IEnvironmentSetupParticipant methods
+
+    def environment_created(self):
+        """Add default wiki pages when environment is created."""
+        self.log.info("Installing default wiki pages")
+        pages_dir = pkg_resources.resource_filename('trac.wiki',
+                                                    'default-pages')
+        with self.env.db_transaction as db:
+            self.load_pages(pages_dir)
+            for page in os.listdir(pages_dir):
+                if page not in ('InterMapTxt', 'SandBox', 'WikiStart'):
+                    db("UPDATE wiki SET readonly='1' WHERE name=%s", (page,))
+
+    def environment_needs_upgrade(self):
+        pass
+
+    def upgrade_environment(self):
+        pass
