@@ -1,27 +1,26 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christian Boos <cboos@edgewall.org>
 
 import re
 
-from genshi.builder import tag
-
 from trac.cache import cached
 from trac.config import ConfigSection
 from trac.core import *
 from trac.util import lazy
+from trac.util.html import tag
 from trac.util.translation import _, N_
 from trac.wiki.api import IWikiChangeListener, IWikiMacroProvider, WikiSystem
 from trac.wiki.parser import WikiParser
@@ -62,7 +61,7 @@ class InterWikiMap(Component):
         return self.interwiki_map[ns.upper()]
 
     def keys(self):
-        return self.interwiki_map.keys()
+        return list(self.interwiki_map)
 
     # Expansion of positional arguments ($1, $2, ...) in URL and title
     def _expand(self, txt, args):
