@@ -1,49 +1,62 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2012 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
-
 from fnmatch import fnmatchcase
 from itertools import groupby
+import fnmatch
 import inspect
+import io
 import os
 import re
-from StringIO import StringIO
-
-from genshi.builder import tag
-from genshi.core import Markup
 
 from trac.core import *
 from trac.resource import (
     Resource, ResourceNotFound, get_resource_name, get_resource_summary,
     get_resource_url
 )
+from trac.util import as_int
 from trac.util.datefmt import format_date, from_utimestamp, user_time
-from trac.util.html import escape, find_element
+from trac.util.html import Markup, escape, find_element, tag
 from trac.util.presentation import separated
 from trac.util.text import unicode_quote, to_unicode, stripws
-from trac.util.translation import _, dgettext, cleandoc_
+from trac.util.translation import _, dgettext, cleandoc_, tag_
+from trac.web.chrome import chrome_resource_path
 from trac.wiki.api import IWikiMacroProvider, WikiSystem, parse_args
 from trac.wiki.formatter import (
-    format_to_html, format_to_oneliner, extract_link, OutlineFormatter
-)
+    MacroError, OutlineFormatter, ProcessorError, extract_link, format_to_html,
+    format_to_oneliner, system_message
+)  # ProcessorError unused, but imported for plugin use.
+from trac.wiki.interwiki import InterWikiMap
 
 
+# TODO: should be moved in .api
 class WikiMacroBase(Component):
-    """Abstract base class for wiki macros."""
+    """Abstract base class for wiki macros and processors.
+
+    On usage error, the `MacroError` or `ProcessorError` exception should be
+    raised, to ensure proper display of the error message in the rendered
+    wiki content.
+
+    The `_description` attribute contains the macro help.
+
+    Set the `hide_from_macro_index` attribute to `True` to prevent
+    displaying help in the macro index (`[[MacroList]]`). If the
+    default value of `False` and the `_description` is empty,
+    "No documentation found" will be displayed.
+    """
 
     implements(IWikiMacroProvider)
     abstract = True
@@ -54,6 +67,9 @@ class WikiMacroBase(Component):
     #: A macro description
     _description = None
 
+    #: Hide from macro index
+    hide_from_macro_index = False
+
     def get_macros(self):
         """Yield the name of the macro based on the class name."""
         name = self.__class__.__name__
@@ -63,6 +79,8 @@ class WikiMacroBase(Component):
 
     def get_macro_description(self, name):
         """Return the subclass's gettext domain and macro description"""
+        if self.hide_from_macro_index:
+            return None
         domain, description = self._domain, self._description
         if description:
             return (domain, description) if domain else description
@@ -79,7 +97,6 @@ class WikiMacroBase(Component):
             (name, self.__class__))
 
 
-
 class TitleIndexMacro(WikiMacroBase):
     _domain = 'messages'
     _description = cleandoc_(
@@ -87,11 +104,16 @@ class TitleIndexMacro(WikiMacroBase):
 
     Accepts a prefix string as parameter: if provided, only pages with names
     that start with the prefix are included in the resulting list. If this
-    parameter is omitted, all pages are listed.
-    If the prefix is specified, a second argument of value `hideprefix`
-    can be given as well, in order to remove that prefix from the output.
+    parameter is omitted, all pages are listed. If the prefix is specified,
+    a second argument of value `hideprefix` can be given as well, in order
+    to remove that prefix from the output.
 
-    Alternate `format` and `depth` named parameters can be specified:
+    The prefix string supports the standard relative-path notation ''when
+    using the macro in a wiki page''. A prefix string starting with `./`
+    will be relative to the current page, and parent pages can be
+    specified using `../`.
+
+    Several named parameters can be specified:
      - `format=compact`: The pages are displayed as comma-separated links.
      - `format=group`: The list of pages will be structured in groups
        according to common prefix. This format also supports a `min=n`
@@ -119,10 +141,9 @@ class TitleIndexMacro(WikiMacroBase):
         args, kw = parse_args(content)
         prefix = args[0].strip() if args else None
         hideprefix = args and len(args) > 1 and args[1].strip() == 'hideprefix'
-        minsize = max(int(kw.get('min', 1)), 1)
+        minsize = _arg_as_int(kw.get('min', 1), 'min', min=1)
         minsize_group = max(minsize, 2)
-        depth = int(kw.get('depth', -1))
-        start = prefix.count('/') if prefix else 0
+        depth = _arg_as_int(kw.get('depth', -1), 'depth', min=-1)
         format = kw.get('format', '')
 
         def parse_list(name):
@@ -132,13 +153,18 @@ class TitleIndexMacro(WikiMacroBase):
         includes = parse_list('include') or ['*']
         excludes = parse_list('exclude')
 
+        wiki = formatter.wiki
+        resource = formatter.resource
+        if prefix and resource and resource.realm == 'wiki':
+            prefix = wiki.resolve_relative_name(prefix, resource.id)
+
+        start = prefix.count('/') if prefix else 0
+
         if hideprefix:
             omitprefix = lambda page: page[len(prefix):]
         else:
             omitprefix = lambda page: page
 
-        wiki = formatter.wiki
-
         pages = sorted(page for page in wiki.get_pages(prefix)
                        if (depth < 0 or depth >= page.count('/') - start)
                        and 'WIKI_VIEW' in formatter.perm('wiki', page)
@@ -188,10 +214,12 @@ class TitleIndexMacro(WikiMacroBase):
                   grouped (path element, page_name) entries
                 - `nodes` is a list of nodes or leaves
             """
+            def keyfn(args):
+                elts, name = args
+                return elts[0] if elts else ''
             groups = []
 
-            for key, grouper in groupby(entries, lambda (elts, name):
-                                                    elts[0] if elts else ''):
+            for key, grouper in groupby(entries, keyfn):
                 # remove key from path_elements in grouped entries for further
                 # grouping
                 grouped_entries = [(path_elements[1:], page_name)
@@ -232,13 +260,15 @@ class TitleIndexMacro(WikiMacroBase):
                 - `page` is a page name (if one exists for that node)
                 - `nodes` is a list of nodes or leaves
             """
+            def keyfn(args):
+                elts, name = args
+                return elts[0] if elts else ''
             groups = []
 
-            for key, grouper in groupby(entries, lambda (elts, name):
-                                                    elts[0] if elts else ''):
-                grouped_entries  = [e for e in grouper]
-                sub_entries  = [e for e in grouped_entries if len(e[0]) > 1]
-                key_entries  = [e for e in grouped_entries if len(e[0]) == 1]
+            for key, grouper in groupby(entries, keyfn):
+                grouped_entries = [e for e in grouper]
+                sub_entries = [e for e in grouped_entries if len(e[0]) > 1]
+                key_entries = [e for e in grouped_entries if len(e[0]) == 1]
                 key_entry = key_entries[0] if key_entries else None
                 key_page = key_entry[1] if key_entries else None
 
@@ -280,7 +310,7 @@ class TitleIndexMacro(WikiMacroBase):
             'group': lambda p: render_group(tree_group(split_pages_group(p))),
             'hierarchy': lambda p: render_hierarchy(
                                     tree_hierarchy(split_pages_hierarchy(p))),
-            }.get(format, None)
+            }.get(format)
 
         if transform:
             titleindex = transform(pages)
@@ -322,7 +352,7 @@ class RecentChangesMacro(WikiMacroBase):
     def expand_macro(self, formatter, name, content):
         args, kw = parse_args(content)
         prefix = args[0].strip() if args else None
-        limit = int(args[1].strip()) if len(args) > 1 else None
+        limit = _arg_as_int(args[1].strip(), min=1) if len(args) > 1 else None
         group = kw.get('group', 'date')
 
         sql = """SELECT name, max(version) AS max_version,
@@ -365,12 +395,11 @@ class RecentChangesMacro(WikiMacroBase):
             for date, entries in entries_per_date)
 
         if group == 'date':
-            out = ((tag.h3(date), tag.ul(entries))
+            out = ((tag.h3(date, class_='section'), tag.ul(entries))
                    for date, entries in items_per_date)
         else:
-            out = tag.ul((entries)
-                         for date, entries in items_per_date)
-        return tag.div(out)
+            out = tag.ul(entries for date, entries in items_per_date)
+        return tag.div(out, class_="wikipage")
 
 
 class PageOutlineMacro(WikiMacroBase):
@@ -409,10 +438,12 @@ class PageOutlineMacro(WikiMacroBase):
             if len(argv) > 0:
                 depth = argv[0]
                 if '-' in depth:
-                    min_depth, max_depth = [int(d)
-                                            for d in depth.split('-', 1)]
+                    min_depth, max_depth = \
+                        [_arg_as_int(d, min=min_depth, max=max_depth)
+                         for d in depth.split('-', 1)]
                 else:
-                    min_depth = max_depth = int(depth)
+                    min_depth = max_depth = \
+                        _arg_as_int(depth, min=min_depth, max=max_depth)
                 if len(argv) > 1:
                     title = argv[1].strip()
                     for arg in argv[2:]:
@@ -424,14 +455,14 @@ class PageOutlineMacro(WikiMacroBase):
 
         # TODO: - integrate the rest of the OutlineFormatter directly here
         #       - use formatter.wikidom instead of formatter.source
-        out = StringIO()
+        out = io.StringIO()
         oformatter = OutlineFormatter(self.env, formatter.context)
         oformatter.format(formatter.source, out, max_depth, min_depth,
                           shorten=not inline)
         outline = Markup(out.getvalue())
 
         if title:
-            outline = tag.h4(title) + outline
+            outline = tag.h4(title, class_='section') + outline
         if not inline:
             outline = tag.div(outline, class_='wiki-toc')
         elif not numbered:
@@ -454,16 +485,19 @@ class ImageMacro(WikiMacroBase):
      * `file` to refer to a local attachment named 'file'. This only works from
        within that wiki page or a ticket.
 
-    Also, the file specification may refer to repository files, using the
-    `source:file` syntax (`source:file@rev` works also).
-
-    Files can also be accessed with a direct URLs; `/file` for a
-    project-relative, `//file` for a server-relative, or `http://server/file`
-    for absolute location of the file.
+    The file specification may also refer to:
+     * repository files, using the `source:file` syntax
+       (`source:file@rev` works also).
+     * files, using direct URLs: `/file` for a project-relative,
+       `//file` for a server-relative, or `http://server/file` for
+       absolute location. An InterWiki prefix may be used.
+     * embedded data using the
+       [http://tools.ietf.org/html/rfc2397 rfc2397] `data` URL scheme,
+       provided the URL is enclosed in quotes.
 
     The remaining arguments are optional and allow configuring the attributes
     and style of the rendered `<img>` element:
-     * digits and unit are interpreted as the size (ex. 120, 25%)
+     * digits and unit are interpreted as the size (ex. 120px, 25%)
        for the image
      * `right`, `left`, `center`, `top`, `bottom` and `middle` are interpreted
        as the alignment for the image (alternatively, the first three can be
@@ -471,12 +505,16 @@ class ImageMacro(WikiMacroBase):
      * `link=some TracLinks...` replaces the link to the image source by the
        one specified using a TracLinks. If no value is specified, the link is
        simply removed.
+     * `inline` specifies that the content generated be an inline XHTML
+       element. By default, inline content is not generated, therefore images
+       won't be rendered in section headings and other one-line content.
      * `nolink` means without link to image source (deprecated, use `link=`)
      * `key=value` style are interpreted as HTML attributes or CSS style
        indications for the image. Valid keys are:
         * align, valign, border, width, height, alt, title, longdesc, class,
           margin, margin-(left,right,top,bottom), id and usemap
         * `border`, `margin`, and `margin-`* can only be a single number
+          (units are pixels).
         * `margin` is superseded by `center` which uses auto margins
 
     Examples:
@@ -504,27 +542,34 @@ class ImageMacro(WikiMacroBase):
     """)
 
     def is_inline(self, content):
-        return True
+        args = [stripws(arg) for arg
+                             in self._split_args_re.split(content or '')[1::2]]
+        return 'inline' in args
 
-    _split_filespec_re = re.compile(r''':(?!(?:[^"':]|[^"']:[^'"])+["'])''')
+    _split_re = r'''((?:[^%s"']|"[^"]*"|'[^']*')+)'''
+    _split_args_re = re.compile(_split_re % ',')
+    _split_filespec_re = re.compile(_split_re % ':')
+    _size_re = re.compile('[0-9]+(%|px)?$')
+    _attr_re = re.compile('(align|valign|border|width|height|alt'
+                          '|margin(?:-(?:left|right|top|bottom))?'
+                          '|title|longdesc|class|id|usemap)=(.+)')
+    _quoted_re = re.compile("(?:[\"'])(.*)(?:[\"'])$")
 
     def expand_macro(self, formatter, name, content):
-        # args will be null if the macro is called without parenthesis.
-        if not content:
+        args = None
+        if content:
+            content = stripws(content)
+            # parse arguments
+            # we expect the 1st argument to be a filename (filespec)
+            args = [stripws(arg) for arg
+                                 in self._split_args_re.split(content)[1::2]]
+        if not args:
             return ''
-        # parse arguments
-        # we expect the 1st argument to be a filename (filespec)
-        args = content.split(',')
         # strip unicode white-spaces and ZWSPs are copied from attachments
         # section (#10668)
-        filespec = stripws(args.pop(0))
+        filespec = args.pop(0)
 
         # style information
-        size_re = re.compile('[0-9]+(%|px)?$')
-        attr_re = re.compile('(align|valign|border|width|height|alt'
-                             '|margin(?:-(?:left|right|top|bottom))?'
-                             '|title|longdesc|class|id|usemap)=(.+)')
-        quoted_re = re.compile("(?:[\"'])(.*)(?:[\"'])$")
         attr = {}
         style = {}
         link = ''
@@ -539,8 +584,8 @@ class ImageMacro(WikiMacroBase):
         except Exception:
             browser_links = []
         while args:
-            arg = stripws(args.pop(0))
-            if size_re.match(arg):
+            arg = args.pop(0)
+            if self._size_re.match(arg):
                 # 'width' keyword
                 attr['width'] = arg
             elif arg == 'nolink':
@@ -561,35 +606,42 @@ class ImageMacro(WikiMacroBase):
             elif arg in ('top', 'bottom', 'middle'):
                 style['vertical-align'] = arg
             else:
-                match = attr_re.match(arg)
+                match = self._attr_re.match(arg)
                 if match:
                     key, val = match.groups()
                     if (key == 'align' and
                             val in ('left', 'right', 'center')) or \
-                        (key == 'valign' and \
+                        (key == 'valign' and
                             val in ('top', 'middle', 'bottom')):
                         args.append(val)
                     elif key in ('margin-top', 'margin-bottom'):
-                        style[key] = ' %dpx' % int(val)
+                        style[key] = ' %dpx' % _arg_as_int(val, key, min=1)
                     elif key in ('margin', 'margin-left', 'margin-right') \
                              and 'display' not in style:
-                        style[key] = ' %dpx' % int(val)
+                        style[key] = ' %dpx' % _arg_as_int(val, key, min=1)
                     elif key == 'border':
-                        style['border'] = ' %dpx solid' % int(val)
+                        style['border'] = ' %dpx solid' % _arg_as_int(val, key)
                     else:
-                        m = quoted_re.search(val) # unquote "..." and '...'
+                        m = self._quoted_re.search(val)  # unquote "..." and '...'
                         if m:
                             val = m.group(1)
-                        attr[str(key)] = val # will be used as a __call__ kwd
+                        attr[str(key)] = val  # will be used as a __call__ kwd
 
+        if self._quoted_re.match(filespec):
+            filespec = filespec.strip('\'"')
         # parse filespec argument to get realm and id if contained.
-        parts = [i.strip('''['"]''')
-                 for i in self._split_filespec_re.split(filespec)]
+        parts = [i.strip('\'"')
+                 for i in self._split_filespec_re.split(filespec)[1::2]]
+        realm = parts[0] if parts else None
         url = raw_url = desc = None
         attachment = None
-        if (parts and parts[0] in ('http', 'https', 'ftp')): # absolute
+        interwikimap = InterWikiMap(self.env)
+        if realm in ('http', 'https', 'ftp', 'data'):  # absolute
             raw_url = url = filespec
             desc = url.rsplit('?')[0]
+        elif realm in interwikimap:
+            url, desc = interwikimap.url(realm, ':'.join(parts[1:]))
+            raw_url = url
         elif filespec.startswith('//'):       # server-relative
             raw_url = url = filespec[1:]
             desc = url.rsplit('?')[0]
@@ -613,7 +665,7 @@ class ImageMacro(WikiMacroBase):
                 attachment = Resource(realm, id).child('attachment', filename)
         elif len(parts) == 2:
             realm, filename = parts
-            if realm in browser_links:   # source:path
+            if realm in browser_links:  # source:path
                 # TODO: use context here as well
                 rev = None
                 if '@' in filename:
@@ -622,7 +674,7 @@ class ImageMacro(WikiMacroBase):
                 raw_url = formatter.href.browser(filename, rev=rev,
                                                  format='raw')
                 desc = filespec
-            else: # #ticket:attachment or WikiPage:attachment
+            else:  # #ticket:attachment or WikiPage:attachment
                 # FIXME: do something generic about shorthand forms...
                 realm = None
                 id, filename = parts
@@ -640,27 +692,36 @@ class ImageMacro(WikiMacroBase):
                 if realm:
                     attachment = Resource(realm, id).child('attachment',
                                                            filename)
-        elif len(parts) == 1: # it's an attachment of the current resource
+        elif len(parts) == 1:  # it's an attachment of the current resource
             attachment = formatter.resource.child('attachment', filespec)
         else:
-            raise TracError(_("No filespec given"))
-        if attachment and 'ATTACHMENT_VIEW' in formatter.perm(attachment):
-            url = get_resource_url(self.env, attachment, formatter.href)
-            raw_url = get_resource_url(self.env, attachment, formatter.href,
-                                       format='raw')
+            return system_message(_("No filespec given"))
+        if attachment:
             try:
                 desc = get_resource_summary(self.env, attachment)
-            except ResourceNotFound, e:
-                raw_url = formatter.href.chrome('common/attachment.png')
+            except ResourceNotFound:
+                link = None
+                raw_url = chrome_resource_path(formatter.context.req,
+                                               'common/attachment.png')
                 desc = _('No image "%(id)s" attached to %(parent)s',
                          id=attachment.id,
                          parent=get_resource_name(self.env, attachment.parent))
+            else:
+                if 'ATTACHMENT_VIEW' in formatter.perm(attachment):
+                    url = get_resource_url(self.env, attachment,
+                                           formatter.href)
+                    raw_url = get_resource_url(self.env, attachment,
+                                               formatter.href, format='raw')
+
         for key in ('title', 'alt'):
-            if desc and not key in attr:
+            if desc and key not in attr:
                 attr[key] = desc
         if style:
             attr['style'] = '; '.join('%s:%s' % (k, escape(v))
                                       for k, v in style.iteritems())
+        if not WikiSystem(self.env).is_safe_origin(raw_url,
+                                                   formatter.context.req):
+            attr['crossorigin'] = 'anonymous'  # avoid password prompt
         result = tag.img(src=raw_url, **attr)
         if link is not None:
             result = tag.a(result, href=link or url,
@@ -682,8 +743,6 @@ class MacroListMacro(WikiMacroBase):
     """)
 
     def expand_macro(self, formatter, name, content):
-        from trac.wiki.formatter import system_message
-
         content = content.strip() if content else ''
         name_filter = content.strip('*')
 
@@ -697,7 +756,7 @@ class MacroListMacro(WikiMacroBase):
                     name_descriptions = [
                         (name, macro_provider.get_macro_description(name))
                         for name in names]
-                except Exception, e:
+                except Exception as e:
                     yield system_message(
                         _("Error: Can't get description for macro %(name)s",
                           name=names[0]), e), names
@@ -718,6 +777,8 @@ class MacroListMacro(WikiMacroBase):
                             else:
                                 descr = format_to_html(
                                     self.env, formatter.context, descr)
+                        elif descr is None:
+                            continue
                         yield descr, [name for name, descr in pairs]
 
         return tag.div(class_='trac-macrolist')(
@@ -725,7 +786,7 @@ class MacroListMacro(WikiMacroBase):
              len(names) > 1 and tag.p(tag.strong(_("Aliases:")),
                                       [tag.code(' [[', alias, ']]')
                                        for alias in names[1:]]) or None,
-             description or tag.em(_("Sorry, no documentation found")))
+             description or tag.em(_("No documentation found")))
             for description, names in sorted(get_macro_descr(),
                                              key=lambda item: item[1][0]))
 
@@ -735,38 +796,56 @@ class TracIniMacro(WikiMacroBase):
     _description = cleandoc_(
     """Produce documentation for the Trac configuration file.
 
-    Typically, this will be used in the TracIni page.
-    Optional arguments are a configuration section filter,
+    Typically, this will be used in the TracIni page. The macro accepts
+    two ordered arguments and two named arguments.
+
+    The ordered arguments are a configuration section filter,
     and a configuration option name filter: only the configuration
     options whose section and name start with the filters are output.
+
+    The named arguments can be specified:
+
+     section :: a glob-style filtering on the section names
+     option  :: a glob-style filtering on the option names
     """)
 
     def expand_macro(self, formatter, name, content):
         from trac.config import ConfigSection, Option
-        section_filter = key_filter = ''
+
         args, kw = parse_args(content)
-        if args:
-            section_filter = args.pop(0).strip()
-        if args:
-            key_filter = args.pop(0).strip()
+        filters = {}
+        for name, index in (('section', 0), ('option', 1)):
+            pattern = kw.get(name, '').strip()
+            if pattern:
+                filters[name] = fnmatch.translate(pattern)
+                continue
+            prefix = args[index].strip() if index < len(args) else ''
+            if prefix:
+                filters[name] = re.escape(prefix)
+        has_option_filter = 'option' in filters
+        for name in ('section', 'option'):
+            filters[name] = re.compile(filters[name], re.IGNORECASE).match \
+                            if name in filters \
+                            else lambda v: True
+        section_filter = filters['section']
+        option_filter = filters['option']
 
-        def getdoc(option_or_section):
-            doc = to_unicode(option_or_section.__doc__)
-            if doc:
-                doc = dgettext(option_or_section.doc_domain, doc)
-            return doc
-
-        registry = ConfigSection.get_registry(self.compmgr)
-        sections = dict((name, getdoc(section))
-                        for name, section in registry.iteritems()
-                        if name.startswith(section_filter))
-
-        registry = Option.get_registry(self.compmgr)
+        section_registry = ConfigSection.get_registry(self.compmgr)
+        option_registry = Option.get_registry(self.compmgr)
         options = {}
-        for (section, key), option in registry.iteritems():
-            if section.startswith(section_filter):
+        for (section, key), option in option_registry.iteritems():
+            if section_filter(section) and option_filter(key):
                 options.setdefault(section, {})[key] = option
-                sections.setdefault(section, '')
+        if not has_option_filter:
+            for section in section_registry:
+                if section_filter(section):
+                    options.setdefault(section, {})
+        for section in options:
+            options[section] = sorted(options[section].itervalues(),
+                                      key=lambda option: option.name)
+        sections = [(section, section_registry[section].doc
+                              if section in section_registry else '')
+                    for section in sorted(options)]
 
         def default_cell(option):
             default = option.default
@@ -776,21 +855,27 @@ class TracIniMacro(WikiMacroBase):
             else:
                 return tag.td(_("(no default)"), class_='nodefault')
 
+        def options_table(section, options):
+            if options:
+                return tag.table(class_='wiki')(
+                    tag.tbody(
+                        tag.tr(
+                            tag.td(tag.a(tag.code(option.name),
+                                         class_='tracini-option',
+                                         href='#%s-%s-option' %
+                                              (section, option.name))),
+                            tag.td(format_to_html(self.env, formatter.context,
+                                                  option.doc)),
+                            default_cell(option),
+                            id='%s-%s-option' % (section, option.name),
+                            class_='odd' if idx % 2 else 'even')
+                     for idx, option in enumerate(options)))
+
         return tag.div(class_='tracini')(
             (tag.h3(tag.code('[%s]' % section), id='%s-section' % section),
              format_to_html(self.env, formatter.context, section_doc),
-             tag.table(class_='wiki')(tag.tbody(
-                 tag.tr(tag.td(tag.tt(option.name)),
-                        tag.td(format_to_oneliner(
-                            self.env, formatter.context, getdoc(option))),
-                        default_cell(option),
-                        class_='odd' if idx % 2 else 'even')
-                 for idx, option in \
-                    enumerate(sorted(options.get(section, {}).itervalues(),
-                                     key=lambda o: o.name))
-                 if option.name.startswith(key_filter))))
-            for section, section_doc in sorted(sections.iteritems()))
-
+             options_table(section, options.get(section)))
+            for section, section_doc in sections)
 
 
 class KnownMimeTypesMacro(WikiMacroBase):
@@ -818,17 +903,16 @@ class KnownMimeTypesMacro(WikiMacroBase):
         return tag.div(class_='mimetypes')(
             tag.table(class_='wiki')(
                 tag.thead(tag.tr(
-                    tag.th(_("MIME Types")), # always use plural
+                    tag.th(_("MIME Types")),  # always use plural
                     tag.th(tag.a("WikiProcessors",
                                  href=formatter.context.href.wiki(
                                      'WikiProcessors'))))),
                 tag.tbody(
-                    tag.tr(tag.th(tag.tt(mime_type),
+                    tag.tr(tag.th(tag.code(mime_type),
                                   style="text-align: left"),
                            tag.td(tag.code(
                                ' '.join(sorted(mime_types[mime_type])))))
-                    for mime_type in sorted(mime_types.keys()))))
-
+                    for mime_type in sorted(mime_types))))
 
 
 class TracGuideTocMacro(WikiMacroBase):
@@ -837,9 +921,10 @@ class TracGuideTocMacro(WikiMacroBase):
     """Display a table of content for the Trac guide.
 
     This macro shows a quick and dirty way to make a table-of-contents
-    for the Help/Guide. The table of contents will contain the Trac* and
-    WikiFormatting pages, and can't be customized. Search for TocMacro for a
-    a more customizable table of contents.
+    for the !Help/Guide. The table of contents will contain the Trac* and
+    WikiFormatting pages, and can't be customized. See the
+    [https://trac-hacks.org/wiki/TocMacro TocMacro] for a more customizable
+    table of contents.
     """)
 
     TOC = [('TracGuide',                    'Index'),
@@ -851,7 +936,7 @@ class TracGuideTocMacro(WikiMacroBase):
            ('TracAdmin',                    'Administration'),
            ('TracBackup',                   'Backup'),
            ('TracLogging',                  'Logging'),
-           ('TracPermissions' ,             'Permissions'),
+           ('TracPermissions',              'Permissions'),
            ('TracWiki',                     'The Wiki'),
            ('WikiFormatting',               'Wiki Formatting'),
            ('TracTimeline',                 'Timeline'),
@@ -886,3 +971,12 @@ class TracGuideTocMacro(WikiMacroBase):
                            class_=(prefix+ref == curpage and 'active'))
                     for ref, title in self.TOC]),
             class_='wiki-toc')
+
+
+def _arg_as_int(val, key=None, min=None, max=None):
+    int_val = as_int(val, None, min=min, max=max)
+    if int_val is None:
+        raise MacroError(tag_("Invalid macro argument %(expr)s",
+                              expr=tag.code("%s=%s" % (key, val))
+                                   if key else tag.code(val)))
+    return int_val
