@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2018 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -19,8 +19,6 @@
 import pkg_resources
 import re
 
-from genshi.builder import tag
-
 from trac.attachment import AttachmentModule, Attachment
 from trac.config import IntOption
 from trac.core import *
@@ -31,12 +29,13 @@ from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import as_int, get_reporter_id
 from trac.util.datefmt import from_utimestamp, to_utimestamp
+from trac.util.html import tag
 from trac.util.text import shorten_line
 from trac.util.translation import _, tag_
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
 from trac.web.api import HTTPBadRequest, IRequestHandler
-from trac.web.chrome import (Chrome, INavigationContributor,
-                             ITemplateProvider, add_ctxtnav, add_link,
+from trac.web.chrome import (Chrome, INavigationContributor, ITemplateProvider,
+                             accesskey, add_ctxtnav, add_link,
                              add_notice, add_script, add_stylesheet,
                              add_warning, prevnext_nav, web_context)
 from trac.wiki.api import IWikiPageManipulator, WikiSystem, validate_page_name
@@ -62,6 +61,8 @@ class WikiModule(Component):
         """Default height of the textarea on the wiki edit page.
         (//Since 1.1.5//)""")
 
+    START_PAGE = property(lambda self: WikiSystem.START_PAGE)
+    TITLE_INDEX_PAGE = property(lambda self: WikiSystem.TITLE_INDEX_PAGE)
     PAGE_TEMPLATES_PREFIX = 'PageTemplates/'
     DEFAULT_PAGE_TEMPLATE = 'DefaultPage'
 
@@ -80,13 +81,14 @@ class WikiModule(Component):
         return 'wiki'
 
     def get_navigation_items(self, req):
-        if 'WIKI_VIEW' in req.perm(self.realm, 'WikiStart'):
+        if 'WIKI_VIEW' in req.perm(self.realm, self.START_PAGE):
             yield ('mainnav', 'wiki',
-                   tag.a(_("Wiki"), href=req.href.wiki(), accesskey=1))
+                   tag.a(_("Wiki"), href=req.href.wiki(),
+                         accesskey=accesskey(req, 1)))
         if 'WIKI_VIEW' in req.perm(self.realm, 'TracGuide'):
             yield ('metanav', 'help',
                    tag.a(_("Help/Guide"), href=req.href.wiki('TracGuide'),
-                         accesskey=6))
+                         accesskey=accesskey(req, 6)))
 
     # IPermissionRequestor methods
 
@@ -106,8 +108,10 @@ class WikiModule(Component):
 
     def process_request(self, req):
         action = req.args.get('action', 'view')
-        pagename = req.args.get('page', 'WikiStart')
-        version = req.args.getint('version')
+        pagename = req.args.get('page', self.START_PAGE)
+        version = None
+        if req.args.get('version'):  # Allow version to be empty
+            version = req.args.getint('version')
         old_version = req.args.getint('old_version')
 
         if pagename.startswith('/') or pagename.endswith('/') or \
@@ -254,8 +258,15 @@ class WikiModule(Component):
     def _do_edit_comment(self, req, page):
         req.perm(page.resource).require('WIKI_ADMIN')
 
+        redirect_to = req.args.get('redirect_to')
+        version = old_version = None
+        if redirect_to == 'diff':
+            version = page.version
+            old_version = version - 1
+        redirect_href = req.href.wiki(page.name, action=redirect_to,
+                                      version=version, old_version=old_version)
         if 'cancel' in req.args:
-            req.redirect(req.href.wiki(page.name, action='history'))
+            req.redirect(redirect_href)
 
         new_comment = req.args.get('new_comment')
 
@@ -263,7 +274,7 @@ class WikiModule(Component):
         add_notice(req, _("The comment of version %(version)s of the page "
                           "%(name)s has been updated.",
                           version=page.version, name=page.name))
-        req.redirect(req.href.wiki(page.name, action='history'))
+        req.redirect(redirect_href)
 
     def _do_delete(self, req, page):
         req.perm(page.resource).require('WIKI_DELETE')
@@ -277,7 +288,7 @@ class WikiModule(Component):
         with self.env.db_transaction:
             if version and old_version and version > old_version:
                 # delete from `old_version` exclusive to `version` inclusive:
-                for v in range(old_version, version):
+                for v in xrange(old_version, version):
                     page.delete(v + 1)
             else:
                 # only delete that `version`, or the whole page if `None`
@@ -332,7 +343,7 @@ class WikiModule(Component):
                 author = get_reporter_id(req)
                 comment = u'[wiki:"%s@%d" %s] \u2192 [wiki:"%s"].' % (
                           new_name, old_version, old_name, new_name)
-                redirection.save(author, comment, req.remote_addr)
+                redirection.save(author, comment)
 
         add_notice(req, _("The page %(old_name)s has been renamed to "
                           "%(new_name)s.", old_name=old_name,
@@ -350,14 +361,13 @@ class WikiModule(Component):
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
 
-        if 'WIKI_ADMIN' in req.perm(page.resource):
+        if 'WIKI_CHANGE_READONLY' in req.perm(page.resource):
             # Modify the read-only flag if it has been changed and the user is
             # WIKI_ADMIN
             page.readonly = int('readonly' in req.args)
 
         try:
-            page.save(get_reporter_id(req, 'author'), req.args.get('comment'),
-                      req.remote_addr)
+            page.save(get_reporter_id(req, 'author'), req.args.get('comment'))
         except TracError:
             add_warning(req, _("Page not modified, showing latest version."))
             return self._render_view(req, page)
@@ -384,7 +394,7 @@ class WikiModule(Component):
         num_versions = 0
         new_date = None
         old_date = None
-        for v, t, author, comment, ipnr in page.get_history():
+        for v, t, author, comment in page.get_history():
             if (v <= version or what == 'page') and new_date is None:
                 new_date = t
             if (v <= old_version and what == 'multiple' or
@@ -403,7 +413,7 @@ class WikiModule(Component):
         if version is not None:
             data.update({'new_version': version, 'old_version': old_version})
         self._wiki_ctxtnav(req, page)
-        return 'wiki_delete.html', data, None
+        return 'wiki_delete.html', data
 
     def _render_confirm_rename(self, req, page, new_name=None):
         req.perm(page.resource).require('WIKI_RENAME')
@@ -411,7 +421,7 @@ class WikiModule(Component):
         data = self._page_data(req, page, 'rename')
         data['new_name'] = new_name if new_name is not None else page.name
         self._wiki_ctxtnav(req, page)
-        return 'wiki_rename.html', data, None
+        return 'wiki_rename.html', data
 
     def _render_diff(self, req, page):
         if not page.exists:
@@ -432,15 +442,14 @@ class WikiModule(Component):
         req.perm(latest_page.resource).require('WIKI_VIEW')
         new_version = page.version
 
-        date = author = comment = ipnr = None
+        date = author = comment = None
         num_changes = 0
         prev_version = next_version = None
-        for version, t, a, c, i in latest_page.get_history():
+        for version, t, a, c in latest_page.get_history():
             if version == new_version:
                 date = t
                 author = a or 'anonymous'
                 comment = c or '--'
-                ipnr = i or ''
             else:
                 if version < new_version:
                     num_changes += 1
@@ -476,18 +485,18 @@ class WikiModule(Component):
 
         data = self._page_data(req, page, 'diff')
         data.update({
-            'change': {'date': date, 'author': author, 'ipnr': ipnr,
-                       'comment': comment},
+            'change': {'date': date, 'author': author, 'comment': comment},
             'new_version': new_version, 'old_version': old_version,
             'latest_version': latest_page.version,
             'num_changes': num_changes,
             'longcol': 'Version', 'shortcol': 'v',
             'changes': changes,
             'diff': diff_data,
+            'can_edit_comment': 'WIKI_ADMIN' in req.perm(page.resource),
         })
         prevnext_nav(req, _("Previous Change"), _("Next Change"),
                      _("Wiki History"))
-        return 'wiki_diff.html', data, None
+        return 'wiki_diff.html', data
 
     def _render_editor(self, req, page, action='edit', has_collision=False):
         if has_collision:
@@ -506,24 +515,30 @@ class WikiModule(Component):
         if 'text' in req.args:
             page.text = req.args.get('text')
         elif 'template' in req.args:
-            template = self.PAGE_TEMPLATES_PREFIX + req.args.get('template')
+            template = req.args.get('template')
+            template = template[1:] if template.startswith('/') \
+                                    else self.PAGE_TEMPLATES_PREFIX + template
             template_page = WikiPage(self.env, template)
             if template_page and template_page.exists and \
                     'WIKI_VIEW' in req.perm(template_page.resource):
                 page.text = template_page.text
         elif 'version' in req.args:
-            version = req.args.getint('version')
-            old_page = WikiPage(self.env, page.name, version)
-            req.perm(page.resource).require('WIKI_VIEW')
-            page.text = old_page.text
-            comment = _("Reverted to version %(version)s.", version=version)
+            version = None
+            if req.args.get('version'):  # Allow version to be empty
+                version = req.args.as_int('version')
+            if version is not None:
+                old_page = WikiPage(self.env, page.name, version)
+                req.perm(page.resource).require('WIKI_VIEW')
+                page.text = old_page.text
+                comment = _("Reverted to version %(version)s.",
+                            version=version)
         if action in ('preview', 'diff'):
             page.readonly = 'readonly' in req.args
 
         author = get_reporter_id(req, 'author')
         defaults = {'editrows': str(self.default_edit_area_height)}
-        prefs = dict((key, req.session.get('wiki_%s' % key, defaults.get(key)))
-                     for key in ('editrows', 'sidebyside'))
+        prefs = {key: req.session.get('wiki_%s' % key, defaults.get(key))
+                 for key in ('editrows', 'sidebyside')}
 
         if 'from_editor' in req.args:
             sidebyside = req.args.get('sidebyside') or None
@@ -554,10 +569,7 @@ class WikiModule(Component):
             'sidebyside': sidebyside,
             'scroll_bar_pos': req.args.get('scroll_bar_pos', ''),
             'diff': None,
-            'attachments': AttachmentModule(self.env).attachment_data(context),
-            'show_readonly_checkbox': ReadonlyWikiPolicy.__name__ in
-                                      self.config.get('trac',
-                                                      'permission_policies')
+            'attachments': AttachmentModule(self.env).attachment_data(context)
         })
         if action in ('diff', 'merge'):
             old_text = original_text.splitlines() if original_text else []
@@ -573,14 +585,15 @@ class WikiModule(Component):
         self._wiki_ctxtnav(req, page)
         Chrome(self.env).add_wiki_toolbars(req)
         Chrome(self.env).add_auto_preview(req)
-        add_script(req, 'common/js/folding.js')
-        return 'wiki_edit.html', data, None
+        add_script(req, 'common/js/wiki.js')
+        return 'wiki_edit.html', data
 
     def _render_edit_comment(self, req, page):
         req.perm(page.resource).require('WIKI_ADMIN')
         data = self._page_data(req, page, 'edit_comment')
+        data.update({'redirect_to': req.args.get('redirect_to', 'history')})
         self._wiki_ctxtnav(req, page)
-        return 'wiki_edit_comment.html', data, None
+        return 'wiki_edit_comment.html', data
 
     def _render_history(self, req, page):
         """Extract the complete history for a given page.
@@ -594,13 +607,12 @@ class WikiModule(Component):
         data = self._page_data(req, page, 'history')
 
         history = []
-        for version, date, author, comment, ipnr in page.get_history():
+        for version, date, author, comment in page.get_history():
             history.append({
                 'version': version,
                 'date': date,
                 'author': author,
-                'comment': comment,
-                'ipnr': ipnr
+                'comment': comment or ''
             })
         data.update({
             'history': history,
@@ -609,7 +621,7 @@ class WikiModule(Component):
         })
         add_ctxtnav(req, _("Back to %(wikipage)s", wikipage=page.name),
                     req.href.wiki(page.name))
-        return 'history_view.html', data, None
+        return 'history_view.html', data
 
     def _render_view(self, req, page):
         version = page.resource.version
@@ -624,7 +636,7 @@ class WikiModule(Component):
                          conversion.in_mimetype)
 
         data = self._page_data(req, page)
-        if page.name == 'WikiStart':
+        if page.name == self.START_PAGE:
             data['title'] = ''
 
         ws = WikiSystem(self.env)
@@ -637,7 +649,7 @@ class WikiModule(Component):
             formatter = OneLinerFormatter(self.env, context)
             if '/' in page.name:
                 parts = page.name.split('/')
-                for i in range(len(parts) - 2, -1, -1):
+                for i in xrange(len(parts) - 2, -1, -1):
                     name = '/'.join(parts[:i] + [parts[-1]])
                     if not ws.has_page(name):
                         higher.append(ws._format_link(formatter, 'wiki',
@@ -714,21 +726,23 @@ class WikiModule(Component):
             'text': text,
             'latest_version': latest_page.version,
             'attachments': AttachmentModule(self.env).attachment_data(context),
+            'start_page': self.START_PAGE,
             'default_template': self.DEFAULT_PAGE_TEMPLATE,
             'templates': templates,
             'version': version,
             'higher': higher, 'related': related,
             'resourcepath_template': 'wiki_page_path.html',
+            'fullwidth': req.session.get('wiki_fullwidth'),
         })
-        add_script(req, 'common/js/folding.js')
-        return 'wiki_view.html', data, None
+        add_script(req, 'common/js/wiki.js')
+        return 'wiki_view.html', data
 
     def _wiki_ctxtnav(self, req, page):
         """Add the normal wiki ctxtnav entries."""
-        if 'WIKI_VIEW' in req.perm('wiki', 'WikiStart'):
-            add_ctxtnav(req, _("Start Page"), req.href.wiki('WikiStart'))
-        if 'WIKI_VIEW' in req.perm('wiki', 'TitleIndex'):
-            add_ctxtnav(req, _("Index"), req.href.wiki('TitleIndex'))
+        if 'WIKI_VIEW' in req.perm('wiki', self.START_PAGE):
+            add_ctxtnav(req, _("Start Page"), req.href.wiki(self.START_PAGE))
+        if 'WIKI_VIEW' in req.perm('wiki', self.TITLE_INDEX_PAGE):
+            add_ctxtnav(req, _("Index"), req.href.wiki(self.TITLE_INDEX_PAGE))
         if page.exists:
             add_ctxtnav(req, _("History"), req.href.wiki(page.name,
                                                          action='history'))
@@ -809,9 +823,12 @@ class WikiModule(Component):
             yield result
 
 
-class ReadonlyWikiPolicy(Component):
-    """Permission policy for the wiki that enforces the read-only attribute
-    for wiki pages."""
+class DefaultWikiPolicy(Component):
+    """Default permission policy for the wiki system.
+
+    Wiki pages with the read-only attribute require `WIKI_ADMIN` to delete,
+    modify or rename the page.
+    """
 
     implements(IPermissionPolicy)
 
@@ -820,8 +837,10 @@ class ReadonlyWikiPolicy(Component):
     # IPermissionPolicy methods
 
     def check_permission(self, action, username, resource, perm):
-        if resource and resource.realm == self.realm and \
-                action in ('WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_RENAME'):
-            page = WikiPage(self.env, resource)
-            if page.readonly and 'WIKI_ADMIN' not in perm(resource):
-                return False
+        if resource and resource.realm == self.realm:
+            if action == 'WIKI_CHANGE_READONLY':
+                return 'WIKI_ADMIN' in perm(resource)
+            if action in ('WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_RENAME'):
+                page = WikiPage(self.env, resource)
+                if page.readonly and 'WIKI_ADMIN' not in perm(resource):
+                    return False
