@@ -1,31 +1,31 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012 Edgewall Software
+# Copyright (C) 2012-2019 Edgewall Software
 # Copyright (C) 2006-2011, Herbert Valerio Riedel <hvr@gnu.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
-import os
 import codecs
-from collections import deque
-from contextlib import contextmanager
-import cStringIO
-from functools import partial
+import contextlib
+import io
+import os
 import re
-from subprocess import Popen, PIPE
-from threading import Lock
 import weakref
+from collections import deque
+from functools import partial
+from subprocess import PIPE
+from threading import Lock
 
 from trac.core import TracBaseError
 from trac.util import terminate
-from trac.util.compat import close_fds
+from trac.util.compat import Popen, close_fds
 from trac.util.datefmt import time_now
 from trac.util.text import to_unicode
 
@@ -85,13 +85,6 @@ def _unquote(path):
     return path
 
 
-def _close_proc_pipes(proc):
-    if proc:
-        for f in (proc.stdin, proc.stdout, proc.stderr):
-            if f:
-                f.close()
-
-
 class GitCore(object):
     """Low-level wrapper around git executable"""
 
@@ -143,9 +136,8 @@ class GitCore(object):
 
         #print("DEBUG:", git_cmd, cmd_args, file=sys.stderr)
 
-        p = self.__pipe(git_cmd, stdout=PIPE, stderr=PIPE, *cmd_args)
-        stdout_data, stderr_data = p.communicate()
-        _close_proc_pipes(p)
+        with self.__pipe(git_cmd, *cmd_args) as p:
+            stdout_data, stderr_data = p.communicate()
         if self.__log and (p.returncode != 0 or stderr_data):
             self.__log.debug('%s exits with %d, dir: %r, args: %s %r, '
                              'stderr: %r', self.__git_bin, p.returncode,
@@ -154,14 +146,14 @@ class GitCore(object):
         return stdout_data
 
     def cat_file_batch(self):
-        return self.__pipe('cat-file', '--batch', stdin=PIPE, stdout=PIPE)
+        return self.__pipe('cat-file', '--batch')
 
     def log_pipe(self, *cmd_args):
-        return self.__pipe('log', stdout=PIPE, *cmd_args)
+        return self.__pipe('log', *cmd_args)
 
     def __getattr__(self, name):
         if name[0] == '_' or name in ['cat_file_batch', 'log_pipe']:
-            raise AttributeError, name
+            raise AttributeError(name)
         return partial(self.__execute, name.replace('_','-'))
 
     __is_sha_pat = re.compile(r'[0-9A-Fa-f]*$')
@@ -311,7 +303,7 @@ class Storage(object):
         assert len(rev) >= 4
         #assert GitCore.is_sha(rev)
         srev_key = int(rev[:4], 16)
-        assert srev_key >= 0 and srev_key <= 0xffff
+        assert 0 <= srev_key <= 0xffff
         return srev_key
 
     @staticmethod
@@ -395,28 +387,31 @@ class Storage(object):
             self._fs_to_unicode = self._fs_from_unicode = lambda s: s
 
         # simple sanity checking
-        __git_file_path = partial(os.path.join, git_dir)
-        control_files = ['HEAD', 'objects', 'refs']
-        control_files_exist = \
-            lambda p: all(map(os.path.exists, map(p, control_files)))
-        if not os.path.exists(__git_file_path()):
-            raise GitError("Path '%s' does not exist" % git_dir)
-        if not control_files_exist(__git_file_path):
-            __git_file_path = partial(os.path.join, git_dir, '.git')
-            if os.path.exists(__git_file_path()) and \
-                    control_files_exist(__git_file_path):
-                git_dir = __git_file_path()
+        try:
+            os.listdir(git_dir)
+        except EnvironmentError as e:
+            self._raise_not_readable(git_dir, e)
+        if not self._control_files_exist(git_dir):
+            dot_git_dir = os.path.join(git_dir, '.git')
+            try:
+                os.listdir(dot_git_dir)
+            except EnvironmentError:
+                missing = True
             else:
+                if self._control_files_exist(dot_git_dir):
+                    missing = False
+                    git_dir = dot_git_dir
+                else:
+                    missing = True
+            if missing:
                 raise GitError("Git control files not found in '%s'" % git_dir)
 
         # at least, check that the HEAD file is readable
-        head_file = os.path.join(git_dir, 'HEAD')
         try:
-            with open(head_file, 'rb'):
+            with open(os.path.join(git_dir, 'HEAD'), 'rb'):
                 pass
-        except IOError as e:
-            raise GitError("Make sure the Git repository '%s' is readable: %s"
-                           % (git_dir, to_unicode(e)))
+        except EnvironmentError as e:
+            self._raise_not_readable(git_dir, e)
 
         self.repo = GitCore(git_dir, git_bin, log, git_fs_encoding)
         self.repo_path = git_dir
@@ -426,7 +421,9 @@ class Storage(object):
 
     def _cleanup_proc(self, proc):
         if proc:
-            _close_proc_pipes(proc)
+            for f in (proc.stdin, proc.stdout, proc.stderr):
+                if f:
+                    f.close()
             terminate(proc)
             proc.wait()
 
@@ -484,10 +481,9 @@ class Storage(object):
         def _rev_reuse(rev):
             return revs_seen.setdefault(rev, rev)
 
-        refs = dict((refname, _rev_reuse(rev))
-                    for refname, rev in refs.iteritems())
-        head_revs = set(rev for refname, rev in refs.iteritems()
-                            if refname.startswith('refs/heads/'))
+        refs = {refname: _rev_reuse(rev) for refname, rev in refs.iteritems()}
+        head_revs = {rev for refname, rev in refs.iteritems()
+                         if refname.startswith('refs/heads/')}
         rev_list = [map(_rev_reuse, line.split())
                     for line in self.repo.rev_list('--parents', '--topo-order',
                                                    '--all').splitlines()]
@@ -548,7 +544,7 @@ class Storage(object):
         rheads_seen = None
 
         # convert sdb either to dict or array depending on size
-        tmp = [()] * (max(new_sdb.keys()) + 1) if len(new_sdb) > 5000 else {}
+        tmp = [()] * (max(new_sdb) + 1) if len(new_sdb) > 5000 else {}
         try:
             while True:
                 k, v = new_sdb.popitem()
@@ -589,12 +585,19 @@ class Storage(object):
         """returns list of (local) branches, with active (= HEAD) one being
         the first item
         """
+        def fn(args):
+            name, rev, head = args
+            return not head, name
         branches = sorted(((self._fs_to_unicode(name), rev, head)
                            for name, rev, head
-                           in self.rev_cache.iter_branches()),
-                          key=lambda (name, rev, head): (not head, name))
+                           in self.rev_cache.iter_branches()), key=fn)
         return [(name, rev) for name, rev, head in branches]
 
+    def get_refs(self):
+        for refname, rev in self.rev_cache.refs_dict.iteritems():
+            if refname != 'HEAD':
+                yield refname, rev
+
     def get_commits(self):
         return self.rev_cache.rev_dict
 
@@ -753,9 +756,9 @@ class Storage(object):
 
         # find a shortened id for which rev doesn't conflict with
         # the other ones from srevs
-        crevs = srevs - set([rev])
+        crevs = srevs - {rev}
 
-        for l in range(min_len+1, 40):
+        for l in xrange(min_len+1, 40):
             srev = rev[:l]
             if srev not in [ r[:l] for r in crevs ]:
                 return srev
@@ -802,6 +805,7 @@ class Storage(object):
 
             meta, fname = l.split('\t', 1)
             _mode, _type, _sha, _size = meta.split()
+            _mode = int(_mode, 8)
             _size = None if _size == '-' else int(_size)
             return _mode, _type, _sha, _size, self._fs_to_unicode(fname)
 
@@ -835,7 +839,7 @@ class Storage(object):
             return result[0], dict(result[1])
 
     def get_file(self, sha):
-        return cStringIO.StringIO(self.cat_file('blob', str(sha)))
+        return io.BytesIO(self.cat_file('blob', str(sha)))
 
     def get_obj_size(self, sha):
         sha = str(sha)
@@ -893,7 +897,7 @@ class Storage(object):
         with self.__rev_cache_lock:
             return self._refresh_rev_cache(force=True)
 
-    @contextmanager
+    @contextlib.contextmanager
     def get_historian(self, sha, base_path):
         p = []
         change = {}
@@ -935,7 +939,7 @@ class Storage(object):
                 return change[path]
             except KeyError:
                 next_path[:] = [path]
-                return gen.next()
+                return next(gen)
 
         try:
             yield historian
@@ -1027,7 +1031,7 @@ class Storage(object):
         iterate = iter_entry(result)
 
         def next_entry():
-            return iterate.next()
+            return next(iterate)
 
         if not tree1:
             # if only one tree-sha is given on commandline,
@@ -1044,6 +1048,8 @@ class Storage(object):
             values = entry[1:].split(' ')
             assert len(values) == 5
             old_mode, new_mode, old_sha, new_sha, change = values
+            old_mode = int(old_mode, 8)
+            new_mode = int(new_mode, 8)
             change = change[:1]
             old_path = self._fs_to_unicode(next_entry())
             new_path = None
@@ -1051,3 +1057,15 @@ class Storage(object):
                 new_path = self._fs_to_unicode(next_entry())
             yield (old_mode, new_mode, old_sha, new_sha, change, old_path,
                    new_path)
+
+    def _raise_not_readable(self, git_dir, e):
+        raise GitError("Make sure the Git repository '%s' is readable: %s"
+                       % (git_dir, to_unicode(e)))
+
+    def _control_files_exist(self, git_dir):
+        for name in ('HEAD', 'objects', 'refs'):
+            if not os.path.exists(os.path.join(git_dir, name)):
+                self.logger.debug("Missing Git control file '%s' in '%s'",
+                                  name, git_dir)
+                return False
+        return True
