@@ -1,36 +1,34 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012 Edgewall Software
+# Copyright (C) 2012-2019 Edgewall Software
 # Copyright (C) 2006-2011, Herbert Valerio Riedel <hvr@gnu.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from datetime import datetime
 import itertools
 import os
 
-from genshi.builder import tag
-from genshi.core import Markup
-
+from trac.api import ISystemInfoProvider
 from trac.cache import cached
 from trac.config import BoolOption, IntOption, ListOption, PathOption, Option
 from trac.core import Component, TracError, implements
-from trac.env import ISystemInfoProvider
 from trac.util import shorten_line
 from trac.util.datefmt import FixedOffset, to_timestamp, format_datetime
+from trac.util.html import Markup, tag
 from trac.util.text import to_unicode, exception_to_unicode
 from trac.util.translation import _
 from trac.versioncontrol.api import Changeset, Node, Repository, \
                                     IRepositoryConnector, InvalidRepository,\
                                     NoSuchChangeset, NoSuchNode, \
-                                    IRepositoryProvider
+                                    RepositoryManager, IRepositoryProvider
 from trac.versioncontrol.cache import CACHE_YOUNGEST_REV, CachedRepository, \
                                       CachedChangeset
 from trac.versioncontrol.web_ui import IPropertyRenderer
@@ -72,7 +70,7 @@ class GitCachedRepository(CachedRepository):
             if len(csets) == 1:
                 yield csets[0]
                 continue
-            rev_csets = dict((cset.rev, cset) for cset in csets)
+            rev_csets = {cset.rev: cset for cset in csets}
             while rev_csets:
                 revs = [rev for rev in rev_csets
                             if not any(r in rev_csets
@@ -192,7 +190,7 @@ class GitCachedChangeset(CachedChangeset):
 def _last_iterable(iterable):
     """helper for detecting last iteration in for-loop"""
     i = iter(iterable)
-    v = i.next()
+    v = next(i)
     for nextv in i:
         yield False, v
         v = nextv
@@ -276,7 +274,7 @@ class GitConnector(Component):
             context = context.parent
 
         try:
-            repos = self.env.get_repository(reponame)
+            repos = RepositoryManager(self.env).get_repository(reponame)
 
             if not repos:
                 raise Exception("Repository '%s' not found" % reponame)
@@ -309,13 +307,13 @@ class GitConnector(Component):
         """Wrap `GitRepository` in `CachedRepository`.""")
 
     shortrev_len = IntOption('git', 'shortrev_len', 7,
-        """The length at which a sha1 should be abbreviated to (must
-        be >= 4 and <= 40).
+        """The length at which a sha1 is abbreviated (must be >= 4
+        and <= 40).
         """)
 
     wiki_shortrev_len = IntOption('git', 'wikishortrev_len', 40,
-        """The minimum length of an hex-string for which
-        auto-detection as sha1 is performed (must be >= 4 and <= 40).
+        """The minimum length at which a hex-string in wiki content
+        is formatted as a changeset TracLink (must be >= 4 and <= 40).
         """)
 
     trac_user_rlookup = BoolOption('git', 'trac_user_rlookup', 'false',
@@ -432,7 +430,7 @@ class CsetPropertyRenderer(Component):
             # sha is assumed to be a non-abbreviated 40-chars sha id
             try:
                 reponame = context.resource.parent.id
-                repos = self.env.get_repository(reponame)
+                repos = RepositoryManager(self.env).get_repository(reponame)
                 cset = repos.get_changeset(sha)
                 if label is None:
                     label = repos.display_rev(sha)
@@ -460,8 +458,8 @@ class CsetPropertyRenderer(Component):
                 current_sha = context.resource.id
                 reponame = context.resource.parent.id
 
-                parent_links = intersperse(', ', \
-                    ((sha_link(rev),
+                parent_links = intersperse(', ',
+                                           ((sha_link(rev),
                       ' (',
                       tag.a(_("diff"),
                             title=_("Diff against this parent (show the "
@@ -620,7 +618,7 @@ class GitRepository(Repository):
 
         def get_tree(rev):
             results = self.git.ls_tree(rev, target_path, recursive=True)
-            return dict((result[4], result) for result in results)
+            return {result[4]: result for result in results}
 
         target_path = old_path.strip('/')
         old_tree = get_tree(old_rev)
@@ -961,8 +959,8 @@ class GitwebProjectsRepositoryProvider(Component):
             return
 
         if not os.path.exists(self.projects_list):
-            self.log.warn("The [git] projects_list file was not found at "
-                          "'%s'", self.projects_list)
+            self.log.warning("The [git] projects_list file was not found at "
+                             "'%s'", self.projects_list)
             return
 
         with open(self.projects_list, 'r') as fp:
