@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2018 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Christian Boos <cboos@edgewall.org>
@@ -53,9 +53,9 @@ import weakref
 import posixpath
 from urllib import quote
 
+from trac.api import ISystemInfoProvider
 from trac.config import ListOption, ChoiceOption
 from trac.core import *
-from trac.env import ISystemInfoProvider
 from trac.versioncontrol import Changeset, Node, Repository, \
                                 IRepositoryConnector, InvalidRepository, \
                                 NoSuchChangeset, NoSuchNode
@@ -505,7 +505,7 @@ class SubversionRepository(Repository):
 
     def get_changeset_uid(self, rev):
         """Build a value identifying the `rev` in this repository."""
-        return (self.uuid, rev)
+        return self.uuid, rev
 
     def get_node(self, path, rev=None):
         """Produce a `SubversionNode` from given path and optionally revision
@@ -820,7 +820,7 @@ class SubversionNode(Node):
             return
         pool = Pool(self.pool)
         entries = fs.dir_entries(self.root, self._scoped_path_utf8, pool())
-        for item in entries.keys():
+        for item in entries:
             path = posixpath.join(self.path, _from_svn(item))
             yield SubversionNode(path, self.rev, self.repos, self.pool,
                                  self.root)
@@ -1015,7 +1015,11 @@ class SubversionChangeset(Changeset):
         pool = Pool(self.pool)
         tmp = Pool(pool)
         root = fs.revision_root(self.fs_ptr, self.rev, pool())
-        editor = repos.RevisionChangeCollector(self.fs_ptr, self.rev, pool())
+        try:
+            editor = repos.ChangeCollector(self.fs_ptr, root, pool())
+        except AttributeError:
+            editor = repos.RevisionChangeCollector(self.fs_ptr, self.rev,
+                                                   pool())
         e_ptr, e_baton = delta.make_editor(editor, pool())
         repos.svn_repos_replay(root, e_ptr, e_baton, pool())
 
@@ -1125,15 +1129,15 @@ def DiffChangeEditor():
         # -- svn.delta.Editor callbacks
 
         def open_root(self, base_revision, dir_pool):
-            return ('/', Changeset.EDIT)
+            return '/', Changeset.EDIT
 
         def add_directory(self, path, dir_baton, copyfrom_path, copyfrom_rev,
                           dir_pool):
             self.deltas.append((path, Node.DIRECTORY, Changeset.ADD))
-            return (path, Changeset.ADD)
+            return path, Changeset.ADD
 
         def open_directory(self, path, dir_baton, base_revision, dir_pool):
-            return (path, dir_baton[1])
+            return path, dir_baton[1]
 
         def change_dir_prop(self, dir_baton, name, value, pool):
             path, change = dir_baton
@@ -1270,8 +1274,7 @@ class FileContentStream(object):
                 values[name] = self.KEYWORD_EXPAND_RE.sub(expand, definition)
 
         if values:
-            return dict((key, to_utf8(value))
-                        for key, value in values.iteritems())
+            return {key: to_utf8(value) for key, value in values.iteritems()}
         else:
             return None
 
