@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2013 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # Copyright (C) 2004 Dmitry Yusupov <dmitry_yus@yahoo.com>
 # Copyright (C) 2004 Mark Rowe <mrowe@bluewire.net.nz>
 # Copyright (C) 2005 Bill Soudan <bill@soudan.net>
@@ -13,22 +13,22 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 """
 Import a Bugzilla items into a Trac database.
 
-Requires:  Trac 0.9b1 from http://trac.edgewall.org/
+Requires:  Trac 0.9b1 from https://trac.edgewall.org/
            Python 2.3 from http://www.python.org/
            MySQL >= 3.23 from http://www.mysql.org/
            or PostGreSQL 8.4 from http://www.postgresql.org/
            or SQLite 3 from http://www.sqlite.org/
 
-$Id: bugzilla2trac.py 14963 2016-07-11 20:06:47Z rjollos $
+$Id$
 """
 
 import re
@@ -44,7 +44,7 @@ import re
 #   2.23.3 (2233), 3.04.4 (3044)
 #
 # If you run this script on a version not listed here and it is successful,
-# please file a ticket at http://trac.edgewall.org
+# please file a ticket at https://trac.edgewall.org
 #
 BZ_VERSION = 3044
 
@@ -212,13 +212,12 @@ BUG_NO_REPL = r"#\2"
 ### Script begins here
 ###
 
+import io
 import os
 import sys
 import string
-import StringIO
 
-import MySQLdb
-import MySQLdb.cursors
+import pymysql
 from trac.attachment import Attachment
 from trac.env import Environment
 
@@ -231,7 +230,7 @@ sys.setdefaultencoding('latin1')
 #class Attachment:
 #    def __init__(self, name, data):
 #        self.filename = name
-#        self.file = StringIO.StringIO(data.tostring())
+#        self.file = io.BytesIO(data.tostring())
 
 # simple field translation mapping.  if string not in
 # mapping, just return string, otherwise return value
@@ -374,7 +373,7 @@ class TracDatabase(object):
         return ticket_id
 
     def addTicketCustomField(self, ticket_id, field_name, field_value):
-        if field_value == None:
+        if field_value is None:
             return
         self.env.db_transaction("""
             INSERT INTO ticket_custom (ticket, name, value) VALUES (%s, %s, %s)
@@ -427,7 +426,7 @@ class TracDatabase(object):
             description = a['description']
             id = a['bug_id']
             filename = a['filename']
-            filedata = StringIO.StringIO(a['thedata'])
+            filedata = io.BytesIO(a['thedata'])
             filesize = len(filedata.getvalue())
             time = a['creation_ts']
             print("    ->inserting attachment '%s' for ticket %s -- %s"
@@ -441,7 +440,7 @@ class TracDatabase(object):
 
     def getLoginName(self, cursor, userid):
         if userid not in self.loginNameCache:
-            cursor.execute("SELECT * FROM profiles WHERE userid = %s", (userid))
+            cursor.execute("SELECT * FROM profiles WHERE userid = %s", userid)
             loginName = cursor.fetchall()
 
             if loginName:
@@ -463,17 +462,17 @@ class TracDatabase(object):
             # 2.23.3.
             if BZ_VERSION >= 2233:
                 cursor.execute("SELECT * FROM fielddefs WHERE id = %s",
-                               (fieldid))
+                               fieldid)
             else:
                 cursor.execute("SELECT * FROM fielddefs WHERE fieldid = %s",
-                               (fieldid))
+                               fieldid)
             fieldName = cursor.fetchall()
 
             if fieldName:
                 fieldName = fieldName[0]['name'].lower()
             else:
                 print("WARNING: unknown bugzilla fieldid %d, "
-                      "         recording as unknown" % userid)
+                      "         recording as unknown" % fieldid)
                 fieldName = "unknown"
 
             self.fieldNameCache[fieldid] = fieldName
@@ -487,8 +486,8 @@ def makeWhereClause(fieldName, values, negative=False):
         connector, op = ' AND ', '!='
     else:
         connector, op = ' OR ', '='
-    clause = connector.join(["%s %s '%s'" % (fieldName, op, value)
-                             for value in values])
+    clause = connector.join("%s %s '%s'" % (fieldName, op, value)
+                            for value in values)
     return ' (' + clause + ')'
 
 def convert(_db, _host, _user, _password, _env, _force):
@@ -503,9 +502,9 @@ def convert(_db, _host, _user, _password, _env, _force):
     # init Bugzilla environment
     print("Bugzilla MySQL('%s':'%s':'%s':'%s'): connecting..."
           % (_db, _host, _user, ("*" * len(_password))))
-    mysql_con = MySQLdb.connect(host=_host,
+    mysql_con = pymysql.connect(host=_host,
                 user=_user, passwd=_password, db=_db, compress=1,
-                cursorclass=MySQLdb.cursors.DictCursor,
+                cursorclass=pymysql.cursors.DictCursor,
                 charset='utf8')
     mysql_cur = mysql_con.cursor()
 
@@ -521,16 +520,15 @@ def convert(_db, _host, _user, _password, _env, _force):
             db("DELETE FROM ticket")
             db("DELETE FROM ticket_custom")
             db("DELETE FROM attachment")
-        attachments_dir = os.path.join(os.path.normpath(trac.env.path),
-                                       "attachments")
         # Straight from the Python documentation.
-        for root, dirs, files in os.walk(attachments_dir, topdown=False):
+        for root, dirs, files in os.walk(trac.env.attachments_dir,
+                                         topdown=False):
             for name in files:
                 os.remove(os.path.join(root, name))
             for name in dirs:
                 os.rmdir(os.path.join(root, name))
-        if not os.stat(attachments_dir):
-            os.mkdir(attachments_dir)
+        if not os.stat(trac.env.attachments_dir):
+            os.mkdir(trac.env.attachments_dir)
         print("All tickets cleaned...")
 
 
@@ -747,7 +745,7 @@ def convert(_db, _host, _user, _password, _env, _force):
             added = activity[activityFields['added']]
 
             # statuses and resolutions are in lowercase in trac
-            if field_name == "resolution" or field_name == "bug_status":
+            if field_name in ('resolution', 'bug_status'):
                 removed = removed.lower()
                 added = added.lower()
 
@@ -840,8 +838,8 @@ def convert(_db, _host, _user, _password, _env, _force):
                     break
                 # cc and attachments.isobsolete sometime appear
                 # in different activities with same time
-                if ((field_name == "cc" or field_name == "attachments.isobsolete") \
-                    and oldChange['time'] == ticketChange['time']):
+                if field_name in ('cc', 'attachments.isobsolete') and \
+                        oldChange['time'] == ticketChange['time']:
                     oldChange['newvalue'] += ", " + ticketChange['newvalue']
                     break
             else:
@@ -869,8 +867,8 @@ def convert(_db, _host, _user, _password, _env, _force):
                 keywords.append(kw)
 
         component = bug['component']
-        if (COMPONENTS_FROM_PRODUCTS and \
-            (MAP_ALL_KEYWORDS or component in KEYWORDS_MAPPING)):
+        if (COMPONENTS_FROM_PRODUCTS and
+                (MAP_ALL_KEYWORDS or component in KEYWORDS_MAPPING)):
             kw = KEYWORDS_MAPPING.get(component, component)
             if kw and kw not in keywords:
                 keywords.append(kw)
@@ -893,7 +891,7 @@ def convert(_db, _host, _user, _password, _env, _force):
 
     print("\n8. Importing users and passwords...")
     if BZ_VERSION >= 2164:
-        selectlogins = "SELECT login_name, cryptpassword FROM profiles";
+        selectlogins = "SELECT login_name, cryptpassword FROM profiles"
         if IGNORE_DISABLED_USERS:
             selectlogins = selectlogins + " WHERE disabledtext=''"
         mysql_cur.execute(selectlogins)
