@@ -1,14 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2018 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 
-import StringIO
+import importlib
+import io
 import warnings
 
 from trac.util.text import print_table, printout
@@ -30,13 +31,14 @@ PACKAGES = [
     ("Setuptools",        'setuptools.__version__'),
     ("Pip",               'pip.__version__'),
     ("Wheel",             'wheel.__version__'),
+    ("Jinja2",            'jinja2.__version__'),
     ("Genshi",            '__main__._genshi_version()'),
     ("Babel",             'babel.__version__'),
     ("sqlite3",           ('sqlite3.version',
                            'sqlite3.sqlite_version')),
     ("PySqlite",          ('pysqlite2.dbapi2.version',
                            'pysqlite2.dbapi2.sqlite_version')),
-    ("MySQLdb",           'MySQLdb.__version__'),
+    ("PyMySQL",           'pymysql.__version__'),
     ("Psycopg2",          'psycopg2.__version__'),
     ("SVN bindings",      '__main__._svn_version()'),
     ("Mercurial",         'mercurial.util.version()'),
@@ -71,7 +73,7 @@ def resolve_accessor(accessor):
     try:
         module, attr = accessor.rsplit('.', 1)
         version = attr.replace('()', '')
-        version = getattr(__import__(module, {}, {}, [version]), version)
+        version = getattr(importlib.import_module(module), version)
         if attr.endswith('()'):
             version = version()
         return version
@@ -83,7 +85,7 @@ def shift(prefix, block):
 
 def print_status():
     warnings.filterwarnings('ignore', '', DeprecationWarning) # Twill 0.9...
-    buf = StringIO.StringIO()
+    buf = io.BytesIO()
     package_versions(PACKAGES, buf)
     printout(shift('  ', buf.getvalue()))
 
